"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NavigatorView = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _tsEventEmitter = require("@dominicstop/ts-event-emitter");

var _RNIWrapperView = require("../native_components/RNIWrapperView");

var _RNINavigatorView = require("../native_components/RNINavigatorView");

var _RNINavigatorViewModule = require("../native_modules/RNINavigatorViewModule");

var _NavigatorRouteViewWrapper = require("../wrapper_components/NavigatorRouteViewWrapper");

var _NavigatorUIConstantsContext = require("../context/NavigatorUIConstantsContext");

var _NavigatorViewEventEmitter = require("../types/NavigatorViewEventEmitter");

var Helpers = _interopRequireWildcard(require("../functions/Helpers"));

var _SimpleQueue = require("../functions/SimpleQueue");

var _NavigatorError = require("../functions/NavigatorError");

var _LibraryConstants = require("../constants/LibraryConstants");

var _LibEnv = require("../constants/LibEnv");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

//#region - Type Definitions

/** Represents the current status of the navigator */
var NavStatus;

(function (NavStatus) {
  NavStatus["IDLE"] = "IDLE";
  NavStatus["IDLE_INIT"] = "IDLE_INIT";
  NavStatus["IDLE_ERROR"] = "IDLE_ERROR";
  NavStatus["NAV_PUSHING"] = "NAV_PUSHING";
  NavStatus["NAV_POPPING"] = "NAV_POPPING";
  NavStatus["NAV_REMOVING"] = "NAV_REMOVING";
  NavStatus["NAV_REPLACING"] = "NAV_REPLACING";
  NavStatus["NAV_INSERTING"] = "NAV_INSERTING";
  NavStatus["NAV_UPDATING"] = "NAV_UPDATING";
  NavStatus["UNMOUNTED"] = "UNMOUNTED";
  NavStatus["NAV_ABORT"] = "NAV_ABORT";
})(NavStatus || (NavStatus = {}));

// nav. has been popped before push completed
; // TODO (010): Move to types/InternalTypes

//#endregion
const TIMEOUT_MOUNT = 750;
const TIMEOUT_COMMAND = 1750;
const iOSVersion = parseInt(_reactNative.Platform.Version, 10);
let NAVIGATOR_ID_COUNTER = 0;
let ROUTE_ID_COUNTER = 0;
let nativeRouteKeys = {};

class NavigatorView extends _react.default.PureComponent {
  //#region - Property Declarations

  /** A ref to the `RNINavigatorView` native component. */

  /** Unique identifier for this navigator */

  /** Used for `removeRouteBatchedFromState`*/

  /** Queue for nav. commands: only 1 command is called at a time */
  // note: the key should be the routeID
  //#endregion
  constructor(_props) {
    super(_props);

    _defineProperty(this, "nativeRef", void 0);

    _defineProperty(this, "navigatorID", void 0);

    _defineProperty(this, "navStatus", void 0);

    _defineProperty(this, "emitter", void 0);

    _defineProperty(this, "routesToRemove", void 0);

    _defineProperty(this, "queue", void 0);

    _defineProperty(this, "routeRefMap", {});

    _defineProperty(this, "verifyProps", () => {
      const props = this.props;

      if (props.routes == null) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidProps,
          message: "the 'NavigatorView.routes' prop cannot be empty"
        });
      }

      ;

      if (typeof props.routes !== 'object' || Array.isArray(props.routes)) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidProps,
          message: "the value passed to the 'NavigatorView.routes' prop must be an object"
        });
      }

      ;
      const routeKeys = Object.keys(props.routes);

      if (routeKeys.length === 0) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidProps,
          message: "the 'NavigatorView.routes' prop cannot be an empty object." + ' There must be at least one valid route'
        });
      }

      ;

      for (const routeKey of routeKeys) {
        const routeConfig = props.routes[routeKey]; // skip native routes...

        if (routeConfig.isNativeRoute) continue; // TODO (016): Add user-defined type guard
        // * (x: NavRouteConfigItem): x is NavRouteConfigItemJS

        if (routeConfig.renderRoute == null) {
          throw new _NavigatorError.NavigatorError({
            code: _NavigatorError.NavigatorErrorCodes.invalidProps,
            message: "invalid route config for ".concat(routeKey, " in 'NavigatorView.routes' prop.") + " Missing 'renderRoute' function (all JS routes must have a component to render)"
          });
        }

        ;
      }

      ;
    });

    _defineProperty(this, "isValidRouteKey", routeKey => {
      const {
        routes
      } = this.props;
      const hasNativeRouteKeyMatch = nativeRouteKeys[routeKey] != null;
      const hasRoutesConfigMatch = routes[routeKey] !== undefined;
      return hasNativeRouteKeyMatch || hasRoutesConfigMatch;
    });

    _defineProperty(this, "getInitialRoutes", () => {
      const {
        initialRoutes
      } = this.props;
      const isValid = initialRoutes != null && Array.isArray(initialRoutes);
      const isEmpty = (initialRoutes === null || initialRoutes === void 0 ? void 0 : initialRoutes.length) === 0; // guard: initial validation for `initialRoutes`

      if (!isValid || isEmpty) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidProps,
          message: "Invalid value given to 'initialRoutes' prop"
        });
      }

      ;
      return initialRoutes.map((route, index) => {
        var _Helpers$shallowMerge;

        const config = this.getRouteConfig(route.routeKey); // guard: no matching route config found for `routeKey`

        if (config == null) {
          throw new _NavigatorError.NavigatorError({
            code: _NavigatorError.NavigatorErrorCodes.invalidProps,
            message: "invalid value for 'initialRoutes' prop" + " - no matching route found for 'routeKey': ".concat(route.routeKey, ".") + " All routes must be declared in the routes prop." + " If this is a native route, please add it to the 'routes' prop w/" + " the 'isNativeRoute' property set to 'true'"
          });
        }

        ;
        const routeItem = {
          isNativeRoute: config.isNativeRoute,
          routeID: ROUTE_ID_COUNTER++,
          routeIndex: index,
          routeKey: route.routeKey,
          routeProps: route.routeProps
        };
        return config.isNativeRoute ? routeItem : { ...routeItem,
          routeOptions: (_Helpers$shallowMerge = Helpers.shallowMergeObjects(config.routeOptionsDefault, route.routeOptions)) !== null && _Helpers$shallowMerge !== void 0 ? _Helpers$shallowMerge : undefined
        };
      });
    });

    _defineProperty(this, "getNativeRoutes", () => {
      const {
        activeRoutes
      } = this.state;
      const nativeRoutes = activeRoutes.filter(route => route.isNativeRoute);
      return nativeRoutes.reduce((acc, curr) => {
        acc[curr.routeID] = {
          routeKey: curr.routeKey,
          routeIndex: curr.routeIndex,
          routeProps: curr.routeProps
        };
        return acc;
      }, {});
    });

    _defineProperty(this, "getRoutesToRender", () => {
      // make a copy
      const activeRoutes = [...this.state.activeRoutes];
      return activeRoutes // only return js/react routes
      .filter(route => !route.isNativeRoute) // sort by routeID in asc. order
      .sort((a, b) => a.routeID - b.routeID);
    });

    _defineProperty(this, "getRouteConfig", routeKey => {
      var _nativeRouteKeys;

      const {
        routes
      } = this.props;
      const routeConfig = routes[routeKey];
      const nativeRouteKey = (_nativeRouteKeys = nativeRouteKeys) === null || _nativeRouteKeys === void 0 ? void 0 : _nativeRouteKeys[routeKey];
      const hasNativeRouteKeyMatch = nativeRouteKey != null;
      const isNativeRoute = (routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.isNativeRoute) || hasNativeRouteKeyMatch;
      if (isNativeRoute) return {
        routeKey,
        isNativeRoute: true,
        initialRouteProps: routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.initialRouteProps
      };
      return routeConfig == null ? undefined : {
        routeKey,
        ...routeConfig
      };
    });

    _defineProperty(this, "getLastRouteTransitionDuration", isPushing => {
      var _routeConfig$transiti, _routeConfig$transiti2, _routeConfig$transiti3, _routeConfig$transiti4;

      const {
        activeRoutes
      } = this.state;
      const lastIndex = activeRoutes.length - 1;
      const lastRoute = activeRoutes[lastIndex]; // if the last route is a native route, then this will be undefined

      const lastRouteRef = this.routeRefMap[lastRoute.routeID];
      if (!lastRouteRef) return 0;
      const routeConfig = lastRouteRef.getRouteOptions();
      return isPushing ? (_routeConfig$transiti = (_routeConfig$transiti2 = routeConfig.transitionConfigPush) === null || _routeConfig$transiti2 === void 0 ? void 0 : _routeConfig$transiti2.duration) !== null && _routeConfig$transiti !== void 0 ? _routeConfig$transiti : 0 : (_routeConfig$transiti3 = (_routeConfig$transiti4 = routeConfig.transitionConfigPop) === null || _routeConfig$transiti4 === void 0 ? void 0 : _routeConfig$transiti4.duration) !== null && _routeConfig$transiti3 !== void 0 ? _routeConfig$transiti3 : 0;
    });

    _defineProperty(this, "configureTransitionOverride", params => {
      var _params$pushConfig, _params$popConfig, _ref;

      let listener;
      const hasTransitionConfig = params.isPushing ? params.pushConfig != null : params.popConfig != null; // the push/pop duration in seconds

      const transitionConfigDuration = params.isPushing ? (_params$pushConfig = params.pushConfig) === null || _params$pushConfig === void 0 ? void 0 : _params$pushConfig.duration : (_params$popConfig = params.popConfig) === null || _params$popConfig === void 0 ? void 0 : _params$popConfig.duration; // note: convert seconds -> ms

      const transitionDuration = 1000 * ((_ref = transitionConfigDuration !== null && transitionConfigDuration !== void 0 ? transitionConfigDuration : this.getLastRouteTransitionDuration(true)) !== null && _ref !== void 0 ? _ref : 0);

      if (transitionDuration > 10000) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidArguments,
          message: "the transition duration of ".concat(transitionDuration, " ms. is too long") + " - reminder: specify duration in seconds (ex: 0.5), not in ms (ex: 500)"
        });
      }

      ;
      return {
        // add some wiggle room
        transitionDuration: transitionDuration + 250,
        setTransition: async () => {
          if (!hasTransitionConfig) return; // set transition

          await Promise.all([// temp bugfix: delay so that pop transition config is set/received
          // from native.
          params.isPushing ? null : Helpers.timeout(100), // temporarily override the last route's push/pop transition
          Helpers.setStateAsync(this, params.isPushing ? {
            transitionConfigPushOverride: params.pushConfig
          } : {
            transitionConfigPopOverride: params.popConfig
          })]);
        },
        resetTransition: async () => {
          var _listener;

          if (!hasTransitionConfig) return;
          (_listener = listener) === null || _listener === void 0 ? void 0 : _listener.unsubscribe(); // remove the push/pop transition override

          await Helpers.setStateAsync(this, {
            transitionConfigPushOverride: undefined,
            transitionConfigPopOverride: undefined
          });
        },
        scheduleReset: () => {
          if (!hasTransitionConfig) return;

          if (params.isPushing) {
            listener = this.emitter.once('onNavRouteDidShow', () => {
              if (this.state.transitionConfigPushOverride == null) return;
              this.setState({
                transitionConfigPushOverride: undefined
              });
            });
          } else {
            this.emitter.once('onNavRouteDidPop', () => {
              if (this.state.transitionConfigPopOverride == null) return;
              this.setState({
                transitionConfigPopOverride: undefined
              });
            });
          }

          ;
        }
      };
    });

    _defineProperty(this, "waitForRoutesToBeAdded", routeItems => {
      // filter out/separate react and native routes
      const [nativeRoutes, reactRoutes] = routeItems.reduce((acc, curr) => {
        acc[curr.isNativeRoute ? 0 : 1].push(curr.routeID);
        return acc;
      }, [[], []]);
      const hasNativeRoutes = nativeRoutes.length > 0;
      return Helpers.promiseWithTimeout(TIMEOUT_MOUNT, Promise.all([// 1. wait for native routes to be init. (if any)
      hasNativeRoutes ? new Promise(resolve => {
        this.emitter.once(_NavigatorViewEventEmitter.NavigatorViewEvents.onSetNativeRoutes, () => {
          resolve();
        });
      }) : null, // 2. wait for react routes to be "received" from native
      ...reactRoutes.map(routeID => new Promise(resolve => {
        this.emitter.once(_NavigatorViewEventEmitter.NavigatorViewEvents.onNavRouteViewAdded, ({
          nativeEvent
        }) => {
          if (nativeEvent.routeID === routeID) {
            resolve();
          }

          ;
        });
      }))]));
    });

    _defineProperty(this, "waitForRoutesToBeRemoved", routeItems => {
      return Promise.all(routeItems.map(routeItem => new Promise(resolve => {
        this.emitter.once(_NavigatorViewEventEmitter.NavigatorViewEvents.onNavRouteDidPop, ({
          nativeEvent
        }) => {
          if (nativeEvent.routeID === routeItem.routeID) {
            resolve();
          }

          ;
        });
      })));
    });

    _defineProperty(this, "syncRoutesFromNative", async () => {
      const state = this.state; // is the current react routes diff. from the native routes?

      let syncStatus = 'sync';
      let activeRoutesNext = [];

      try {
        const activeRoutesNative = await this.getActiveRoutes();
        const activeRoutesMapJS = {};

        for (const route of state.activeRoutes) {
          activeRoutesMapJS[route.routeID] = route;
        }

        ;

        for (let i = 0; i < activeRoutesNative.length; i++) {
          var _state$activeRoutes$i;

          activeRoutesNext[i] = activeRoutesNative[i];

          if (((_state$activeRoutes$i = state.activeRoutes[i]) === null || _state$activeRoutes$i === void 0 ? void 0 : _state$activeRoutes$i.routeID) !== activeRoutesNative[i].routeID) {
            syncStatus = 'diverged';
          }

          ;
        }

        ;

        if (syncStatus === 'diverged') {
          //#region - 🐞 DEBUG 🐛
          _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, syncRoutesFromNative" + " - active routes synced"); //#endregion

          await Helpers.setStateAsync(this, {
            activeRoutes: activeRoutesNext
          });
        }

        ;
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e); //#region - 🐞 DEBUG 🐛

        _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - Error, NavigatorView, syncRoutesFromNative" + " - sync failed w/ error: ".concat(error.message)); //#endregion
        // sync failed

        syncStatus = 'unknown';
      }

      ;
      return {
        results: syncStatus,
        didDivergeFromNative: syncStatus === 'diverged' || syncStatus === 'unknown'
      };
    });

    _defineProperty(this, "removeRouteBatchedFromState", async (params) => {
      if (params) {
        // To prevent too many state updates, the routes to be removed
        // are queued/grouped, and are removed in batches...
        this.routesToRemove.push(params);
      }

      ; // trigger remove if:

      const shouldRemove = () => // - A. there's a param argument
      params != null && // - B. the route to be removed has a valid `routeKey`
      this.isValidRouteKey(params.routeKey) && // - C. there are queued items to remove
      this.routesToRemove.length > 0; // if busy, wait for prev. to finish


      const queue = this.queue.schedule();
      await queue.promise;
      this.setNavStatus(NavStatus.NAV_POPPING);

      while (shouldRemove()) {
        var _params$routeKey, _params$routeIndex;

        // delay, so `routesToRemove` queue gets filled first
        await Helpers.timeout(200); // make a copy of `routesToRemove` and then clear original

        const toBeRemoved = [...this.routesToRemove];
        this.routesToRemove = []; //#region - 🐞 DEBUG 🐛

        _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, removeRouteBatchedFromState" + " - with routeKey: ".concat((_params$routeKey = params === null || params === void 0 ? void 0 : params.routeKey) !== null && _params$routeKey !== void 0 ? _params$routeKey : 'N/A') + " - routeIndex: ".concat((_params$routeIndex = params === null || params === void 0 ? void 0 : params.routeIndex) !== null && _params$routeIndex !== void 0 ? _params$routeIndex : 'N/A') + " - navStatus: ".concat(this.navStatus) + " - current activeRoutes: ".concat(this.state.activeRoutes.length) + " - current routesToRemove: ".concat(toBeRemoved.length) + " - activeRoutes: ".concat(JSON.stringify(this.state.activeRoutes))); //#endregion
        // Remove `routesToRemove` from `activeRoutes`.

        await Helpers.setStateAsync(this, ({
          activeRoutes: prevRoutes
        }) => ({
          activeRoutes: prevRoutes.filter(prevRoute => !toBeRemoved.some(removedRoute => removedRoute.routeKey === prevRoute.routeKey && removedRoute.routeIndex === prevRoute.routeIndex))
        }));
      }

      ;
      this.setNavStatus(NavStatus.IDLE);
      this.queue.dequeue();
    });

    _defineProperty(this, "_handleGetRefToNavigator", () => {
      return this;
    });

    _defineProperty(this, "getActiveRoutes", () => {
      return this.state.activeRoutes;
    });

    _defineProperty(this, "push", async (routeItem, options) => {
      const routeConfig = this.getRouteConfig(routeItem.routeKey);
      const transitionConfig = this.configureTransitionOverride({
        isPushing: true,
        pushConfig: options === null || options === void 0 ? void 0 : options.transitionConfig
      });

      if (!routeConfig) {
        // no matching route config found for `routeItem`
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidArguments,
          message: "invalid 'routeKey': ".concat(routeItem.routeKey)
        });
      }

      ;

      try {
        var _routeConfig$isNative, _options$isAnimated;

        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        const {
          activeRoutes
        } = this.state;
        this.setNavStatus(NavStatus.NAV_PUSHING); // the amount of time to wait for "push" to resolve before rejecting.

        const timeout = Math.max(transitionConfig.transitionDuration, TIMEOUT_COMMAND); // temporarily override the last route's "push" transition

        await transitionConfig.setTransition();
        transitionConfig.scheduleReset(); //#region - 🐞 DEBUG 🐛

        _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, push: add route" + " - with routeKey: ".concat(routeItem.routeKey) + " - pre-push activeRoutes: ".concat(this.state.activeRoutes.length)); //#endregion

        const nextRouteIndex = activeRoutes.length;
        const nextRoute = {
          routeID: ROUTE_ID_COUNTER++,
          routeIndex: nextRouteIndex,
          routeKey: routeItem.routeKey,
          routeProps: routeItem.routeProps,
          routeOptions: routeItem.routeOptions,
          isNativeRoute: (_routeConfig$isNative = routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.isNativeRoute) !== null && _routeConfig$isNative !== void 0 ? _routeConfig$isNative : false
        };
        await Promise.all([// 1. Wait for new route to be added/init
        this.waitForRoutesToBeAdded([nextRoute]), // 2. Append new route to `activeRoutes`
        Helpers.setStateAsync(this, ({
          activeRoutes: prevRoutes
        }) => ({
          activeRoutes: [...prevRoutes, nextRoute]
        }))]); // forward "push" request to native module

        await Helpers.promiseWithTimeout(timeout, _RNINavigatorViewModule.RNINavigatorViewModule.push(Helpers.getNativeNodeHandle(this.nativeRef), nextRoute.routeID, {
          isAnimated: (_options$isAnimated = options === null || options === void 0 ? void 0 : options.isAnimated) !== null && _options$isAnimated !== void 0 ? _options$isAnimated : true
        })); // reset transition override

        transitionConfig.resetTransition(); // finished, start next item

        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        transitionConfig.resetTransition();
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "pop", async options => {
      const {
        activeRoutes
      } = this.state;

      if (activeRoutes.length < 1) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.routeOutOfBounds,
          message: "no routes to pop, the current active route count must be > 1"
        });
      }

      ;
      const transitionConfig = this.configureTransitionOverride({
        isPushing: false,
        popConfig: options === null || options === void 0 ? void 0 : options.transitionConfig
      });

      try {
        var _options$isAnimated2;

        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_POPPING); // the amount of time to wait for "pop" to resolve before rejecting.

        const timeout = Math.max(transitionConfig.transitionDuration, TIMEOUT_COMMAND); // temporarily change the last route's "pop" transition

        await transitionConfig.setTransition();
        transitionConfig.scheduleReset(); // forward "pop" request to native module

        const result = await Helpers.promiseWithTimeout(timeout, _RNINavigatorViewModule.RNINavigatorViewModule.pop(Helpers.getNativeNodeHandle(this.nativeRef), {
          isAnimated: (_options$isAnimated2 = options === null || options === void 0 ? void 0 : options.isAnimated) !== null && _options$isAnimated2 !== void 0 ? _options$isAnimated2 : true
        })); // remove popped route from `activeRoutes`

        await Helpers.setStateAsync(this, prevState => ({
          activeRoutes: prevState.activeRoutes.filter(route => !(route.routeIndex === result.routeIndex && route.routeKey === result.routeKey))
        })); // reset transition override

        await transitionConfig.resetTransition();
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        transitionConfig.resetTransition();
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "popToRoot", async options => {
      const {
        activeRoutes
      } = this.state;

      if (activeRoutes.length < 1) {
        throw new Error("`popToRoot` failed, route count must be > 1");
      }

      ;
      const transitionConfig = this.configureTransitionOverride({
        isPushing: false,
        popConfig: options === null || options === void 0 ? void 0 : options.transitionConfig
      });

      try {
        var _options$isAnimated3;

        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_POPPING); // the amount of time to wait for "pop" to resolve before rejecting.

        const timeout = Math.max(transitionConfig.transitionDuration, TIMEOUT_COMMAND); // temporarily change the last route's "pop" transition

        await transitionConfig.setTransition();
        transitionConfig.scheduleReset(); // forward `popToRoot` request to native module

        await Helpers.promiseWithTimeout(timeout, _RNINavigatorViewModule.RNINavigatorViewModule.popToRoot(Helpers.getNativeNodeHandle(this.nativeRef), {
          isAnimated: (_options$isAnimated3 = options === null || options === void 0 ? void 0 : options.isAnimated) !== null && _options$isAnimated3 !== void 0 ? _options$isAnimated3 : true
        })); // remove popped route from `activeRoutes`

        await Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: [prevState.activeRoutes[0]]
        })); // reset transition override

        await transitionConfig.resetTransition();
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        transitionConfig.resetTransition();
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "removeRoute", async (routeIndex, animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const routeToBeRemoved = activeRoutes[routeIndex];

      if (activeRoutes.length < 1) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.routeOutOfBounds,
          message: "no routes to remove, the current active route count must be > 1"
        });
      }

      ;

      if (routeToBeRemoved == null) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidRouteIndex,
          message: "invalid 'routeIndex' value: ".concat(routeIndex) + " - no matching route could be found for the given value"
        });
      }

      ;

      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_REMOVING);
        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.removeRoute(Helpers.getNativeNodeHandle(this.nativeRef), routeToBeRemoved.routeID, routeToBeRemoved.routeIndex, animated)); // wait a bit so `routeToBeRemoved` can receive the `onRouteDidPop` event

        await Helpers.timeout(100);
        await Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: prevState.activeRoutes // remove route from `activeRoutes`
          .filter(route => route.routeID !== routeToBeRemoved.routeID) // update the route indexes
          .map((route, index) => ({ ...route,
            routeIndex: index
          }))
        }));
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "removeRoutes", async (routeIndices, animated = false) => {
      const {
        activeRoutes
      } = this.state;

      if (routeIndices.length === 0) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidArguments,
          message: "the provided 'routeIndexes' to remove is empty"
        });
      }

      ; // check if `routeIndexes` are valid

      for (const routeIndex of routeIndices) {
        const item = activeRoutes[routeIndex];

        if (item == null) {
          throw new _NavigatorError.NavigatorError({
            code: _NavigatorError.NavigatorErrorCodes.invalidRouteIndex,
            message: "invalid 'routeIndexes' value: ".concat(routeIndex) + " - no matching route found the given value"
          });
        }

        ;
      }

      ;

      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_REMOVING);
        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.removeRoutes(Helpers.getNativeNodeHandle(this.nativeRef), // routes to remove,
        routeIndices.map(routeIndex => ({
          routeIndex,
          routeID: activeRoutes[routeIndex].routeID
        })), animated)); // wait a bit so the routes in `routeIndices` can receive the `onRouteDidPop` event

        await Helpers.timeout(100);
        await Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: prevState.activeRoutes // remove routes from `activeRoutes`
          .filter((_, index) => !routeIndices.some(routeIndex => routeIndex === index)) // update the route indexes
          .map((route, index) => ({ ...route,
            routeIndex: index
          }))
        }));
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "replaceRoute", async (prevRouteIndex, routeItem, animated = false) => {
      var _replacementRouteConf;

      const {
        activeRoutes
      } = this.state;
      const routeToReplace = activeRoutes[prevRouteIndex];
      const replacementRouteConfig = this.getRouteConfig(routeItem.routeKey);

      if (routeToReplace == null) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidRouteIndex,
          message: "invalid 'prevRouteIndex' value: ".concat(prevRouteIndex) + " - no matching route found for the given value"
        });
      }

      ;

      if (replacementRouteConfig == null) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidRouteKey,
          message: "invalid replacement 'routeKey' value: ".concat(routeItem === null || routeItem === void 0 ? void 0 : routeItem.routeKey) + " - no matching route found for the given value"
        });
      }

      ;
      const replacementRoute = {
        routeID: ROUTE_ID_COUNTER++,
        routeKey: routeItem.routeKey,
        routeIndex: prevRouteIndex,
        routeProps: routeItem.routeProps,
        routeOptions: routeItem.routeOptions,
        isNativeRoute: (_replacementRouteConf = replacementRouteConfig === null || replacementRouteConfig === void 0 ? void 0 : replacementRouteConfig.isNativeRoute) !== null && _replacementRouteConf !== void 0 ? _replacementRouteConf : false
      };

      try {
        // if busy, wait for prev. to finish first...
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_REPLACING);
        await Promise.all([// 1. wait for replacement route to be added/init
        this.waitForRoutesToBeAdded([replacementRoute]), // 2. replace route in state
        Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: prevState.activeRoutes.map((route, index) => ({ ...(routeToReplace.routeID === route.routeID ? replacementRoute : route),
            // overwrite new route index...
            routeIndex: index
          }))
        }))]);
        await Helpers.timeout(500); // forward command to native module

        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.replaceRoute(Helpers.getNativeNodeHandle(this.nativeRef), prevRouteIndex, routeToReplace.routeID, replacementRoute.routeID, animated));
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "insertRoute", async (routeItem, atIndex, animated = false) => {
      var _routeConfig$isNative2;

      const state = this.state;
      const routeConfig = this.getRouteConfig(routeItem.routeKey);

      if (!routeConfig) {
        // no matching route config found for `routeItem`
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.invalidRouteKey,
          message: "invalid 'routeKey' value: ".concat(routeItem === null || routeItem === void 0 ? void 0 : routeItem.routeKey) + " - no corresponding route could be for the given value"
        });
      }

      ;

      if (atIndex > state.activeRoutes.length) {
        throw new _NavigatorError.NavigatorError({
          code: _NavigatorError.NavigatorErrorCodes.routeOutOfBounds,
          message: "invalid 'atIndex' value: ".concat(atIndex, " (out of bounds)") + " - the provided 'atIndex' exceeds the total active routes"
        });
      }

      ;
      const nextRoute = {
        routeID: ROUTE_ID_COUNTER++,
        routeKey: routeItem.routeKey,
        routeIndex: atIndex,
        routeProps: routeItem.routeProps,
        routeOptions: routeItem.routeOptions,
        isNativeRoute: (_routeConfig$isNative2 = routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.isNativeRoute) !== null && _routeConfig$isNative2 !== void 0 ? _routeConfig$isNative2 : false
      };

      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_INSERTING); //#region - 🐞 DEBUG 🐛

        _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, insertRoute: add route" + " - with routeKey: ".concat(nextRoute.routeKey) + " - pre-push activeRoutes: ".concat(this.state.activeRoutes.length)); //#endregion

        await Promise.all([// 1. wait for next route to be added/init
        this.waitForRoutesToBeAdded([nextRoute]), // 2. append new route to `activeRoutes`
        Helpers.setStateAsync(this, ({
          activeRoutes: prevRoutes
        }) => ({
          activeRoutes: Helpers.arrayInsert(prevRoutes, atIndex, nextRoute).map((route, index) => ({ ...route,
            routeIndex: index
          }))
        }))]); // forward command to native module

        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.insertRoute(Helpers.getNativeNodeHandle(this.nativeRef), nextRoute.routeID, atIndex, animated)); // finished, start next item

        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "setRoutes", async (transform, animated = false) => {
      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_UPDATING);
        const currentRoutes = [...this.state.activeRoutes];
        const currentRoutesMap = currentRoutes.reduce((acc, curr) => {
          acc[curr.routeID] = { ...curr
          };
          return acc;
        }, {});
        const transformResult = transform(currentRoutes.map(route => ({
          routeID: route.routeID,
          routeKey: route.routeKey
        })));
        const nextRoutes = transformResult.map((route, index) => {
          const prevRoute = route.routeID && currentRoutesMap[route.routeID];

          if (prevRoute) {
            return { // merge old + new route items
              ...prevRoute,
              ...route,
              // assign new routeIndex
              routeIndex: index
            };
          } else {
            var _routeConfig$isNative3;

            const routeConfig = this.getRouteConfig(route.routeKey);

            if (routeConfig == null) {
              throw new _NavigatorError.NavigatorError({
                code: _NavigatorError.NavigatorErrorCodes.invalidRouteKey,
                message: "the transform callback returned an invalid route key." + " - no matching route found for the given 'routeKey': ".concat(route.routeKey)
              });
            }

            ;
            return { ...route,
              // assign a routeID since it's a new route
              routeID: ROUTE_ID_COUNTER++,
              // assign a routeIndex
              routeIndex: index,
              isNativeRoute: (_routeConfig$isNative3 = routeConfig.isNativeRoute) !== null && _routeConfig$isNative3 !== void 0 ? _routeConfig$isNative3 : false
            };
          }

          ;
        }); // get nextRoutes items that aren't mounted/added yet

        const nextRoutesNew = nextRoutes.filter(nextRoute => !currentRoutes.some(currentRoute => currentRoute.routeID === nextRoute.routeID)); //#region - 🐞 DEBUG 🐛

        _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, setRoutes" + " - current routes: ".concat(currentRoutes) + " - next routes: ".concat(nextRoutes) + " - new routes: ".concat(nextRoutesNew)); //#endregion

        await Promise.all([// 1. wait for the new route items to mount (if any)
        this.waitForRoutesToBeAdded(nextRoutesNew), // 2. update the state route items
        Helpers.setStateAsync(this, {
          activeRoutes: nextRoutes
        })]); // forward command to native module

        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.setRoutes(Helpers.getNativeNodeHandle(this.nativeRef), nextRoutes.map(route => route.routeID), animated)); // finished, start next item

        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        const syncStats = await this.syncRoutesFromNative();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted || syncStats.didDivergeFromNative) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "setNavigationBarHidden", async (isHidden, animated) => {
      try {
        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.setNavigationBarHidden(Helpers.getNativeNodeHandle(this.nativeRef), isHidden, animated));
      } catch (e) {
        const error = new _NavigatorError.NavigatorError(e);
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw error;
        }

        ;
      }

      ;
    });

    _defineProperty(this, "replacePreviousRoute", async (routeItem, animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      await this.replaceRoute(lastRouteIndex - 1, routeItem, animated);
    });

    _defineProperty(this, "replaceCurrentRoute", async (routeItem, animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      await this.replaceRoute(lastRouteIndex, routeItem, animated);
    });

    _defineProperty(this, "removePreviousRoute", async (animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      await this.removeRoute(lastRouteIndex - 1, animated);
    });

    _defineProperty(this, "removeAllPrevRoutes", async (animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      const routesToRemove = activeRoutes.slice(0, lastRouteIndex).map((_, index) => index);
      await this.removeRoutes(routesToRemove, animated);
    });

    _defineProperty(this, "sendCustomCommandToNative", async (commandKey, commandData = null) => {
      return await _RNINavigatorViewModule.RNINavigatorViewModule.sendCustomCommandToNative(Helpers.getNativeNodeHandle(this.nativeRef), commandKey, commandData);
    });

    _defineProperty(this, "getNavigatorConstants", async () => {
      try {
        const result = await Helpers.promiseWithTimeout(1000, _RNINavigatorViewModule.RNINavigatorViewModule.getNavigatorConstants(Helpers.getNativeNodeHandle(this.nativeRef)));
        return result;
      } catch (e) {
        throw new _NavigatorError.NavigatorError(e);
      }

      ;
    });

    _defineProperty(this, "dismissModal", async (animated = true) => {
      await _RNINavigatorViewModule.RNINavigatorViewModule.dismissModal(Helpers.getNativeNodeHandle(this.nativeRef), animated);
    });

    _defineProperty(this, "_handleRouteViewWrapperRef", (ref, route) => {
      this.routeRefMap[route.routeID] = ref;
    });

    _defineProperty(this, "_handleOnNavRouteViewAdded", event => {
      if (this.navigatorID !== event.nativeEvent.navigatorID) return; // emit event: nav. route was added to `RNINavigatorView`'s subviews

      this.emitter.emit(_NavigatorViewEventEmitter.NavigatorViewEvents.onNavRouteViewAdded, event);
    });

    _defineProperty(this, "_handleOnSetNativeRoutes", event => {
      if (this.navigatorID !== event.nativeEvent.navigatorID) return; // emit event: route data was set for the native routes

      this.emitter.emit(_NavigatorViewEventEmitter.NavigatorViewEvents.onSetNativeRoutes, event);
    });

    _defineProperty(this, "_handleOnNativeCommandRequest", async ({
      nativeEvent
    }) => {
      if (this.navigatorID !== nativeEvent.navigatorID) return;
      const {
        commandData
      } = nativeEvent; //#region - 🐞 DEBUG 🐛

      _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, _handleOnNativeCommandRequest" + "commandKey: ".concat(nativeEvent.commandData.commandKey) + "commandData: ".concat(JSON.stringify(nativeEvent.commandData))); //#endregion

      switch (commandData.commandKey) {
        // TODO (012): Move to sep. function
        case 'pushViewController':
          try {
            // if busy, wait for prev. to finish
            const queue = this.queue.schedule();
            await queue.promise;
            const {
              activeRoutes
            } = this.state;
            const nextRouteIndex = activeRoutes.length;
            this.setNavStatus(NavStatus.NAV_PUSHING);
            const nextRoute = {
              routeID: commandData.routeID,
              routeKey: commandData.routeKey,
              routeIndex: nextRouteIndex,
              isNativeRoute: true
            };
            await Promise.all([// 1. wait for the native route to be init
            this.waitForRoutesToBeAdded([nextRoute]), // 2. Append new route to `activeRoutes`
            Helpers.setStateAsync(this, ({
              activeRoutes: prevRoutes
            }) => ({
              activeRoutes: [...prevRoutes, nextRoute]
            }))]); // forward "push" request to native module

            await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.push(Helpers.getNativeNodeHandle(this.nativeRef), nextRoute.routeID, {
              isAnimated: commandData.isAnimated
            })); // finished, start next item

            this.setNavStatus(NavStatus.IDLE);
            this.queue.dequeue();
          } catch (e) {
            const error = new _NavigatorError.NavigatorError(e);
            const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
            const syncStats = await this.syncRoutesFromNative();
            this.setNavStatus(NavStatus.IDLE_ERROR);
            this.queue.dequeue();

            if (!wasAborted || syncStats.didDivergeFromNative) {
              throw error;
            }

            ;
          }

          ;
          break;

        case 'push':
          await this.push({
            routeKey: commandData.routeKey,
            routeProps: commandData.routeProps
          }, {
            isAnimated: commandData.isAnimated
          });
          break;

        case 'pop':
          await this.pop({
            isAnimated: commandData.isAnimated
          });
          break;
      }

      ;
    });

    _defineProperty(this, "_handleOnCustomCommandFromNative", event => {
      var _this$props$onCustomC, _this$props;

      if (this.navigatorID !== event.nativeEvent.navigatorID) return;
      (_this$props$onCustomC = (_this$props = this.props).onCustomCommandFromNative) === null || _this$props$onCustomC === void 0 ? void 0 : _this$props$onCustomC.call(_this$props, event);
    });

    _defineProperty(this, "_handleOnNavRouteWillPop", ({
      nativeEvent
    }) => {
      if (this.navigatorID !== nativeEvent.navigatorID) return;
      const {
        onNavRouteWillPop
      } = this.props;
      onNavRouteWillPop && onNavRouteWillPop({
        nativeEvent
      });

      if (!NavigatorViewUtils.isNavStateIdle(this.navStatus)) {
        this.setNavStatus(NavStatus.NAV_ABORT);
      }

      ;
    });

    _defineProperty(this, "_handleOnNavRouteDidPop", ({
      nativeEvent
    }) => {
      if (this.navigatorID !== nativeEvent.navigatorID) return;
      this.emitter.emit('onNavRouteDidPop', {
        nativeEvent
      });
      const {
        onNavRouteDidPop
      } = this.props;
      onNavRouteDidPop && onNavRouteDidPop({
        nativeEvent
      }); //#region - 🐞 DEBUG 🐛

      _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, onNavRouteDidPop" + " - with args, routeKey: ".concat(nativeEvent.routeKey) + " - routeIndex: ".concat(nativeEvent.routeIndex)); //#endregion
      // * A route has been removed either through a tap on the "back" 
      //   button, or through a swipe back gesture.
      // * As such, the route has to be removed from `state.activeRoutes`.

      if (nativeEvent.isUserInitiated) {
        // * In iOS 13+, multiple routes can be removed at once via the back button context menu.
        // * This results in multiple calls to `_handleOnNavRouteDidPop`.
        // * To fix this, there's a special version of `removeRoute` that removes routes in batches.
        if (iOSVersion >= 13) {
          this.removeRouteBatchedFromState({
            routeKey: nativeEvent.routeKey,
            routeIndex: nativeEvent.routeIndex
          });
        } else {
          this.removeRoute(nativeEvent.routeIndex);
        }

        ;
      }

      ;
    });

    _defineProperty(this, "_handleOnUIConstantsDidChange", ({
      nativeEvent
    }) => {
      var _this$props$onUIConst, _this$props2;

      if (this.navigatorID !== nativeEvent.navigatorID) return;
      (_this$props$onUIConst = (_this$props2 = this.props).onUIConstantsDidChange) === null || _this$props$onUIConst === void 0 ? void 0 : _this$props$onUIConst.call(_this$props2, {
        nativeEvent
      });
      this.setState({
        safeAreaInsets: nativeEvent.safeAreaInsets,
        statusBarHeight: nativeEvent.statusBarHeight,
        navigatorSize: nativeEvent.navigatorSize
      });
    });

    _defineProperty(this, "_handleOnNavRouteDidShow", ({
      nativeEvent
    }) => {
      var _this$props$onNavRout, _this$props3;

      if (this.navigatorID !== nativeEvent.navigatorID) return;
      (_this$props$onNavRout = (_this$props3 = this.props).onNavRouteDidShow) === null || _this$props$onNavRout === void 0 ? void 0 : _this$props$onNavRout.call(_this$props3, {
        nativeEvent
      });
      this.emitter.emit('onNavRouteDidShow', {
        nativeEvent
      });
    });

    this.verifyProps();
    this.navigatorID = NAVIGATOR_ID_COUNTER++;
    this.navStatus = NavStatus.IDLE_INIT;
    this.routesToRemove = [];
    this.emitter = new _tsEventEmitter.TSEventEmitter();
    this.queue = new _SimpleQueue.SimpleQueue();
    this.state = {
      activeRoutes: this.getInitialRoutes(),
      transitionConfigPushOverride: undefined,
      transitionConfigPopOverride: undefined,
      safeAreaInsets: undefined,
      statusBarHeight: undefined,
      navigatorSize: undefined
    };
  }

  componentWillUnmount() {
    this.setNavStatus(NavStatus.UNMOUNTED);
    this.queue.clear();
  }

  setNavStatus(navStatus) {
    if (this.navStatus !== NavStatus.UNMOUNTED) {
      //#region - 🐞 DEBUG 🐛
      _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorView, setNavStatus" + " - navigatorID: ".concat(this.navigatorID) + " - next status: ".concat(navStatus) + " - current status: ".concat(this.navStatus) + " - queue.isBusy: ".concat(this.queue.isBusy())); //#endregion

      this.navStatus = navStatus;
    }

    ;
  }

  //#endregion
  _renderRoutes() {
    const props = this.props;
    const state = this.state; // * Get the `state.activeRoutes` sorted by their `routeID`, such that the
    //   routes are ordered from oldest to newest.
    // * Supposedly, when reordering comps, as long as the key of a comp. does
    //   not change, the comps. should be "preserved", but When replacing the 1st 
    //   route (i.e. replacing `state.activeRoutes[0]`), it causes all the other 
    //   routes to disappear.
    // * Specifically, it triggers `componentWillUnmount` for all the other 
    //   active routes (even though there aren't any changes to their props/keys).
    // * This then triggers `vc.didMove(toParent: nil`) in the native side,
    //   which then triggers the cleanup/remove process.
    // * This bug has something to do with the re-ordering of components, causing
    //   the comps. to be "recreated" from scratch.
    // * So this is a temp. workaround to prevent the unnecessary reordering of the
    //   comps., the downside is that the routes need to be sorted on every render...

    const activeRoutes = this.getRoutesToRender();
    const activeRoutesCount = activeRoutes.length;
    const activeRoutesLastIndex = activeRoutesCount - 1;
    return activeRoutes.map(route => {
      var _state$transitionConf;

      // the routes from `activeRoutes` will only ever have js/react routes
      const routeConfig = this.getRouteConfig(route.routeKey);
      const isLast = route.routeIndex === activeRoutesLastIndex;
      const isSecondToLast = route.routeIndex === activeRoutesLastIndex - 1;
      const initialRouteProps = Helpers.shallowMergeObjects(routeConfig.initialRouteProps, props.sharedRouteProps);
      return /*#__PURE__*/_react.default.createElement(_NavigatorRouteViewWrapper.NavigatorRouteViewWrapper, {
        key: "routeID-".concat(route.routeID),
        innerRef: this._handleRouteViewWrapperRef,
        navigatorID: this.navigatorID,
        routeID: route.routeID,
        routeIndex: route.routeIndex,
        routeKey: route.routeKey,
        currentActiveRouteIndex: activeRoutesLastIndex,
        route: route,
        routeConfig: routeConfig // merge routeProps
        ,
        routeProps: Helpers.shallowMergeObjects(initialRouteProps, route.routeProps) // merge routeOptions
        ,
        routeOptionsDefault: Helpers.shallowMergeObjects(routeConfig.routeOptionsDefault, route.routeOptions) // Note: the prev. route's push transition config 
        // determines the next route's push transition
        ,
        transitionConfigPushOverride: isSecondToLast ? (_state$transitionConf = state.transitionConfigPushOverride) !== null && _state$transitionConf !== void 0 ? _state$transitionConf : null : null,
        transitionConfigPopOverride: isLast ? state.transitionConfigPopOverride : null,
        getRefToNavigator: this._handleGetRefToNavigator,
        renderDefaultNavBarLeftItem: props.renderNavBarLeftItem,
        renderDefaultNavBarRightItem: props.renderNavBarRightItem,
        renderDefaultNavBarTitleItem: props.renderNavBarTitleItem
      });
    });
  }

  render() {
    const {
      isInteractivePopGestureEnabled,
      shouldSwizzleRootViewController,
      navBarPrefersLargeTitles,
      navBarAppearance,
      isNavBarTranslucent,
      disableTransparentNavBarScrollEdgeAppearance,
      initialRoutes,
      renderNavBarBackground,
      ...viewProps
    } = this.props;
    const state = this.state;
    const style = viewProps.style == null ? styles.navigatorView : [styles.navigatorView, viewProps.style]; //#region - 🐞 DEBUG 🐛

    _LibEnv.LIB_ENV.debugLogRender && console.log("LOG/JS - NavigatorView, render" + " - navigatorID: ".concat(this.navigatorID)); //#endregion

    return /*#__PURE__*/_react.default.createElement(_RNINavigatorView.RNINavigatorView, _extends({
      ref: r => {
        this.nativeRef = r;
      },
      navigatorID: this.navigatorID,
      style: style // General config
      ,
      isInteractivePopGestureEnabled: isInteractivePopGestureEnabled !== null && isInteractivePopGestureEnabled !== void 0 ? isInteractivePopGestureEnabled : true,
      shouldSwizzleRootViewController: shouldSwizzleRootViewController !== null && shouldSwizzleRootViewController !== void 0 ? shouldSwizzleRootViewController : true,
      disableTransparentNavBarScrollEdgeAppearance: disableTransparentNavBarScrollEdgeAppearance !== null && disableTransparentNavBarScrollEdgeAppearance !== void 0 ? disableTransparentNavBarScrollEdgeAppearance : true,
      nativeRoutes: this.getNativeRoutes(),
      initialRouteKeys: initialRoutes.map(route => route.routeKey) // Navigation Bar customization
      ,
      isNavBarTranslucent: isNavBarTranslucent !== null && isNavBarTranslucent !== void 0 ? isNavBarTranslucent : true,
      navBarPrefersLargeTitles: navBarPrefersLargeTitles !== null && navBarPrefersLargeTitles !== void 0 ? navBarPrefersLargeTitles : true,
      navBarAppearance: navBarAppearance // event handlers: push/pop
      ,
      onNavRouteWillPop: this._handleOnNavRouteWillPop,
      onNavRouteDidPop: this._handleOnNavRouteDidPop,
      onNavRouteViewAdded: this._handleOnNavRouteViewAdded,
      onSetNativeRoutes: this._handleOnSetNativeRoutes,
      onNativeCommandRequest: this._handleOnNativeCommandRequest,
      onCustomCommandFromNative: this._handleOnCustomCommandFromNative,
      onUIConstantsDidChange: this._handleOnUIConstantsDidChange,
      onNavRouteWillShow: this._handleOnNavRouteDidShow,
      onNavRouteDidShow: this._handleOnNavRouteDidShow
    }, viewProps), /*#__PURE__*/_react.default.createElement(_NavigatorUIConstantsContext.NavigatorUIConstantsContext.Provider, {
      value: {
        safeAreaInsets: state.safeAreaInsets,
        statusBarHeight: state.statusBarHeight,
        navigatorSize: state.navigatorSize
      }
    }, this._renderRoutes(), renderNavBarBackground && /*#__PURE__*/_react.default.createElement(_RNIWrapperView.RNIWrapperView, {
      style: styles.navBarBackgroundContainer,
      nativeID: _LibraryConstants.NativeIDKeys.NavBarBackground
    }, renderNavBarBackground())));
  }

}

exports.NavigatorView = NavigatorView;
;
/** Utilities for `NavigatorView` */

class NavigatorViewUtils {
  static sortByRouteIndex(routes) {
    return routes.sort((a, b) => a.routeIndex - b.routeIndex);
  }

  static isNavStateIdle(navStatus) {
    return navStatus === NavStatus.IDLE || navStatus === NavStatus.IDLE_INIT || navStatus === NavStatus.IDLE_ERROR || navStatus === NavStatus.NAV_ABORT;
  }

  static wasAborted(navStatus) {
    return navStatus === NavStatus.UNMOUNTED || navStatus === NavStatus.NAV_ABORT || navStatus === NavStatus.NAV_POPPING;
  }

  static isNavStateBusy(navStatus) {
    return !NavigatorViewUtils.isNavStateIdle(navStatus);
  }

  static getNativeRouteKeys() {
    return new Promise(resolve => {
      _RNINavigatorViewModule.RNINavigatorViewModule.getNativeRouteKeys(keys => {
        resolve(keys.reduce((acc, curr) => {
          acc[curr] = curr;
          return acc;
        }, {}));
      });
    });
  }

}

; // init. `nativeRouteKeys`

NavigatorViewUtils.getNativeRouteKeys().then(keys => {
  nativeRouteKeys = keys;
});

const styles = _reactNative.StyleSheet.create({
  navigatorView: {
    flex: 1
  },
  navBarBackgroundContainer: {
    position: 'absolute'
  }
});
//# sourceMappingURL=NavigatorView.js.map