"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RouteViewEvents = void 0;

var _react = _interopRequireDefault(require("react"));

var _NavigatorRouteViewEventEmitter = require("../types/NavigatorRouteViewEventEmitter");

var _NavigationContext = require("../context/NavigationContext");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** 
 * This component is used to listen to route-related events.
 * 
 * This component is used inside a route, and allows you to listen for the events
 * that are related to that route.
 * 
 * This component is just a "convenient" wrapper component that subscribes to the
 * parent route component's event emitter. The reference to the event emitter is obtained
 * via context.
 */
class RouteViewEvents extends _react.default.Component {
  constructor(props, context) {
    super(props);

    _defineProperty(this, "routerRef", void 0);

    _defineProperty(this, "emitterRef", void 0);

    _defineProperty(this, "_handleOnRouteWillPush", params => {
      var _this$props$onRouteWi, _this$props;

      (_this$props$onRouteWi = (_this$props = this.props).onRouteWillPush) === null || _this$props$onRouteWi === void 0 ? void 0 : _this$props$onRouteWi.call(_this$props, params);
    });

    _defineProperty(this, "_handleOnRouteDidPush", params => {
      var _this$props$onRouteDi, _this$props2;

      (_this$props$onRouteDi = (_this$props2 = this.props).onRouteDidPush) === null || _this$props$onRouteDi === void 0 ? void 0 : _this$props$onRouteDi.call(_this$props2, params);
    });

    _defineProperty(this, "_handleOnRouteWillPop", params => {
      var _this$props$onRouteWi2, _this$props3;

      (_this$props$onRouteWi2 = (_this$props3 = this.props).onRouteWillPop) === null || _this$props$onRouteWi2 === void 0 ? void 0 : _this$props$onRouteWi2.call(_this$props3, params);
    });

    _defineProperty(this, "_handleOnRouteDidPop", params => {
      var _this$props$onRouteDi2, _this$props4;

      (_this$props$onRouteDi2 = (_this$props4 = this.props).onRouteDidPop) === null || _this$props$onRouteDi2 === void 0 ? void 0 : _this$props$onRouteDi2.call(_this$props4, params);
    });

    _defineProperty(this, "_handleOnRouteWillFocus", params => {
      var _this$props$onRouteWi3, _this$props5;

      (_this$props$onRouteWi3 = (_this$props5 = this.props).onRouteWillFocus) === null || _this$props$onRouteWi3 === void 0 ? void 0 : _this$props$onRouteWi3.call(_this$props5, params);
    });

    _defineProperty(this, "_handleOnRouteDidFocus", params => {
      var _this$props$onRouteDi3, _this$props6;

      (_this$props$onRouteDi3 = (_this$props6 = this.props).onRouteDidFocus) === null || _this$props$onRouteDi3 === void 0 ? void 0 : _this$props$onRouteDi3.call(_this$props6, params);
    });

    _defineProperty(this, "_handleOnRouteWillBlur", params => {
      var _this$props$onRouteWi4, _this$props7;

      (_this$props$onRouteWi4 = (_this$props7 = this.props).onRouteWillBlur) === null || _this$props$onRouteWi4 === void 0 ? void 0 : _this$props$onRouteWi4.call(_this$props7, params);
    });

    _defineProperty(this, "_handleOnRouteDidBlur", params => {
      var _this$props$onRouteDi4, _this$props8;

      (_this$props$onRouteDi4 = (_this$props8 = this.props).onRouteDidBlur) === null || _this$props$onRouteDi4 === void 0 ? void 0 : _this$props$onRouteDi4.call(_this$props8, params);
    });

    _defineProperty(this, "_handleOnPressNavBarLeftItem", params => {
      var _this$props$onPressNa, _this$props9;

      (_this$props$onPressNa = (_this$props9 = this.props).onPressNavBarLeftItem) === null || _this$props$onPressNa === void 0 ? void 0 : _this$props$onPressNa.call(_this$props9, params);
    });

    _defineProperty(this, "_handleOnPressNavBarRightItem", params => {
      var _this$props$onPressNa2, _this$props10;

      (_this$props$onPressNa2 = (_this$props10 = this.props).onPressNavBarRightItem) === null || _this$props$onPressNa2 === void 0 ? void 0 : _this$props$onPressNa2.call(_this$props10, params);
    });

    _defineProperty(this, "_handleOnUpdateSearchResults", params => {
      var _this$props$onUpdateS, _this$props11;

      (_this$props$onUpdateS = (_this$props11 = this.props).onUpdateSearchResults) === null || _this$props$onUpdateS === void 0 ? void 0 : _this$props$onUpdateS.call(_this$props11, params);
    });

    _defineProperty(this, "_handleOnSearchBarCancelButtonClicked", params => {
      var _this$props$onSearchB, _this$props12;

      (_this$props$onSearchB = (_this$props12 = this.props).onSearchBarCancelButtonClicked) === null || _this$props$onSearchB === void 0 ? void 0 : _this$props$onSearchB.call(_this$props12, params);
    });

    _defineProperty(this, "_handleOnSearchBarSearchButtonClicked", params => {
      var _this$props$onSearchB2, _this$props13;

      (_this$props$onSearchB2 = (_this$props13 = this.props).onSearchBarSearchButtonClicked) === null || _this$props$onSearchB2 === void 0 ? void 0 : _this$props$onSearchB2.call(_this$props13, params);
    });

    const {
      navigation
    } = context;
    const routeRef = navigation.getRefToRoute();
    this.routerRef = routeRef;
    const routeEmitterRef = routeRef.getEmitterRef();
    this.emitterRef = routeEmitterRef;
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillPush, this._handleOnRouteWillPush);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidPush, this._handleOnRouteDidPush);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillPop, this._handleOnRouteWillPop);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidPop, this._handleOnRouteDidPop);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillFocus, this._handleOnRouteWillFocus);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidFocus, this._handleOnRouteDidFocus);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillBlur, this._handleOnRouteWillBlur);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidBlur, this._handleOnRouteDidBlur);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onPressNavBarLeftItem, this._handleOnPressNavBarLeftItem);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onPressNavBarRightItem, this._handleOnPressNavBarRightItem);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onUpdateSearchResults, this._handleOnUpdateSearchResults);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onSearchBarCancelButtonClicked, this._handleOnSearchBarCancelButtonClicked);
    routeEmitterRef.addListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onSearchBarSearchButtonClicked, this._handleOnSearchBarSearchButtonClicked);
  }

  componentWillUnmount() {
    const routeEmitterRef = this.emitterRef;
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillPush, this._handleOnRouteWillPush);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidPush, this._handleOnRouteDidPush);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillPop, this._handleOnRouteWillPop);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidPop, this._handleOnRouteDidPop);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillFocus, this._handleOnRouteWillFocus);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidFocus, this._handleOnRouteDidFocus);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillBlur, this._handleOnRouteWillBlur);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidBlur, this._handleOnRouteDidBlur);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onPressNavBarLeftItem, this._handleOnPressNavBarLeftItem);
    routeEmitterRef.removeListener(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onPressNavBarRightItem, this._handleOnPressNavBarRightItem);
  }

  //#endregion
  render() {
    return null;
  }

}

exports.RouteViewEvents = RouteViewEvents;

_defineProperty(RouteViewEvents, "contextType", _NavigationContext.NavigationContext);

;
//# sourceMappingURL=RouteViewEvents.js.map