"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RouteViewPortal = void 0;

var _react = _interopRequireDefault(require("react"));

var _CompareRouteOptions = require("../functions/CompareRouteOptions");

var _NavigationContext = require("../context/NavigationContext");

var _LibEnv = require("../constants/LibEnv");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** 
 * This component is used to customize the route's navigation bar.
 * This component is used inside a route, and allows you to update the route's `routeOptions`
 * and provide custom elements to show in the navigation bar.
 * 
 * Reminder: You can only use one `RouteViewPortal` per route.
 * 
 * Through the `routeOptions` prop, you can update the route's `routeTitle` and `backButtonTitle`, 
 * configure the navigation bar items (e.g. like setting the right navigation bar item via 
 * `navBarButtonRightItemsConfig`), or providing a custom view to show in the navigation bar (
 * e.g. you can render a custom title via `renderNavBarTitleItem`, etc.)
 * 
 * Note: The reason why this component is called `RouteViewPortal` is because it's transporting
 * the elements (and props) you pass to this component to the parent route navigator. 
 * In other words, the props and elements you pass to this component are being used/rendered 
 * somewhere else in the view hierarchy. 
 * 
 * This is achieved through context, and as such, you can only use this component if it's a child 
 * of a route. So this component actually renders nothing/`null` and does not affect layout.
 * 
 * Because of this, it has some weird quirks. 
 * For example, there are situations where the parent might re-render (e.g. due to `setState`) 
 * but the components inside the render props (e.g. `renderNavBarLeftItem`) won't update 
 * even though a prop directly depends on a value from the parent's state.
 */
class RouteViewPortal extends _react.default.Component {
  constructor(props, context) {
    super(props);

    _defineProperty(this, "routeRef", void 0);

    _defineProperty(this, "componentDidUpdate", prevProps => {
      const nextProps = this.props;
      const didRouteOptionsChange = !_CompareRouteOptions.CompareRouteOptions.unwrapAndCompare(prevProps.routeOptions, nextProps.routeOptions);

      if (didRouteOptionsChange) {
        // updating the `routeOptions` will trigger a re-render/update...
        this.routeRef.setPortalRouteOptions(nextProps.routeOptions); //#region - 🐞 DEBUG 🐛

        _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - RouteViewPortal, componentDidUpdate" + " - didRouteOptionsChange: ".concat(didRouteOptionsChange ? 'true' : 'false')); //#endregion
      } else {
        // `routeOptions` did not change, trigger a re-render 
        this.routeRef.updateRouteComponents();
      }

      ;
    });

    _defineProperty(this, "componentWillUnmount", () => {
      this.routeRef.setPortalRouteOptions(null);
    });

    const {
      navigation
    } = context;
    const routeRef = navigation.getRefToRoute();
    this.routeRef = routeRef;
    routeRef.setRouteViewPortalRef(this);

    if (props.routeOptions != null) {
      routeRef.setPortalRouteOptions(props.routeOptions);
    }

    ;
  }

  render() {
    return null;
  }

}

exports.RouteViewPortal = RouteViewPortal;

_defineProperty(RouteViewPortal, "contextType", _NavigationContext.NavigationContext);

;
//# sourceMappingURL=RouteViewPortal.js.map