"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setStateAsync = setStateAsync;
exports.timeout = timeout;
exports.promiseWithTimeout = promiseWithTimeout;
exports.isClassComponent = isClassComponent;
exports.lastElement = lastElement;
exports.arrayInsert = arrayInsert;
exports.addOptional = addOptional;
exports.shallowMergeObjects = shallowMergeObjects;
exports.getNativeNodeHandle = getNativeNodeHandle;

var _reactNative = require("react-native");

/** wrapper func for setState that returns a promise */
// eslint-disable-next-line consistent-this
function setStateAsync(that, newState) {
  return new Promise(resolve => {
    that.setState(newState, () => {
      resolve();
    });
  });
}

;
/** wrapper for timeout that returns a promise */

function timeout(ms) {
  return new Promise(resolve => {
    const timeoutID = setTimeout(() => {
      clearTimeout(timeoutID);
      resolve();
    }, ms);
  });
}

;
/** Wraps a promise that will reject if not not resolved in <ms> milliseconds */

function promiseWithTimeout(ms, promise) {
  // Create a promise that rejects in <ms> milliseconds
  const timeoutPromise = new Promise((_, reject) => {
    const timeoutID = setTimeout(() => {
      clearTimeout(timeoutID);
      reject("Promise timed out in ".concat(ms, " ms."));
    }, ms);
  }); // Returns a race between our timeout and the passed in promise

  return Promise.race([promise, timeoutPromise]);
}

;

function isClassComponent(component) {
  var _component$prototype;

  return !!(typeof component === 'function' && component !== null && component !== void 0 && (_component$prototype = component.prototype) !== null && _component$prototype !== void 0 && _component$prototype.isReactComponent);
}

;

function lastElement(array) {
  if (!array) return undefined;
  return array[array.length - 1];
}

;

function arrayInsert(array, index, newItem) {
  return [// part of the array before the specified index
  ...array.slice(0, index), // inserted item
  newItem, // part of the array after the specified index
  ...array.slice(index)];
}

;

function addOptional(a, b) {
  return { ...a,
    ...b
  };
}

;

function shallowMergeObjects(a, b) {
  if (a == null && b == null) return null;else if (a != null && b == null) return a;else if (a == null && b != null) return b;else return { ...a,
    ...b
  };
}

;

function getNativeNodeHandle(nativeRef) {
  const nodeHandle = (0, _reactNative.findNodeHandle)(nativeRef);

  if (nodeHandle == null) {
    throw new Error('Unable to get the node handle for the native ref.');
  }

  ;
  return nodeHandle;
}

;
//# sourceMappingURL=Helpers.js.map