"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NavigatorRouteContentWrapper = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _CompareMisc = require("../functions/CompareMisc");

var _CompareUtilities = require("../functions/CompareUtilities");

var _LibraryConstants = require("../constants/LibraryConstants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Wraps the react element returned from `props.renderRouteContent()`
 * render prop in `NavigatorRouteView._renderRouteContents`.
 * * The purpose of this component is to stop unnecessary re-renders from
 *   propagating down to the "route content".
 */
class NavigatorRouteContentWrapper extends _react.default.Component {
  shouldComponentUpdate(nextProps) {
    const prevProps = this.props; // when `routePropsUpdateIndex` increments, then `routeProps` changed

    const didRoutePropsChange = prevProps.routePropsUpdateIndex !== nextProps.routePropsUpdateIndex; // only update when in focus or the route props changed

    const shouldUpdate = didRoutePropsChange || nextProps.isInFocus;
    return shouldUpdate && ( // compare whether there's a `renderRouteHeader` or not
    prevProps.hasRouteHeader !== nextProps.hasRouteHeader || // compare `automaticallyAddHorizontalSafeAreaInsets`
    prevProps.automaticallyAddHorizontalSafeAreaInsets !== nextProps.automaticallyAddHorizontalSafeAreaInsets // compare `safeAreaInsets`
    || !_CompareMisc.CompareEdgeInsets.unwrapAndCompare(prevProps.safeAreaInsets, nextProps.safeAreaInsets) || !_CompareUtilities.CompareUtilities.unwrapAndShallowCompareObject(prevProps.routeContainerStyle, nextProps.routeContainerStyle));
  }

  render() {
    var _safeAreaInsets$left, _safeAreaInsets$right;

    const props = this.props;
    const safeAreaInsets = props.safeAreaInsets;
    const routeContainerStyle = { ...(props.automaticallyAddHorizontalSafeAreaInsets && {
        paddingLeft: (_safeAreaInsets$left = safeAreaInsets === null || safeAreaInsets === void 0 ? void 0 : safeAreaInsets.left) !== null && _safeAreaInsets$left !== void 0 ? _safeAreaInsets$left : 0,
        paddingRight: (_safeAreaInsets$right = safeAreaInsets === null || safeAreaInsets === void 0 ? void 0 : safeAreaInsets.right) !== null && _safeAreaInsets$right !== void 0 ? _safeAreaInsets$right : 0
      })
    };
    return /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: [styles.routeContentContainer, routeContainerStyle, props.routeContainerStyle],
      nativeID: _LibraryConstants.NativeIDKeys.RouteContent
    }, this.props.children);
  }

}

exports.NavigatorRouteContentWrapper = NavigatorRouteContentWrapper;
;

const styles = _reactNative.StyleSheet.create({
  routeContentContainer: { ..._reactNative.StyleSheet.absoluteFillObject,
    overflow: 'hidden'
  }
});
//# sourceMappingURL=NavigatorRouteContentWrapper.js.map