function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { StyleSheet } from 'react-native';
import { TSEventEmitter } from '@dominicstop/ts-event-emitter';
import { NavigatorRouteViewEvents } from '../types/NavigatorRouteViewEventEmitter';
import { RouteComponentsWrapper } from '../wrapper_components/RouteComponentsWrapper';
import { NavigatorRouteContentWrapper } from '../wrapper_components/NavigatorRouteContentWrapper';
import { RouteHeaderWrapper } from '../wrapper_components/RouteHeaderWrapper';
import { RNINavigatorRouteView } from '../native_components/RNINavigatorRouteView';
import { RNINavigatorRouteViewModule } from '../native_modules/RNINavigatorRouteViewModule';
import * as Helpers from '../functions/Helpers';
import { CompareRouteTransitionPopConfig, CompareRouteTransitionPushConfig } from '../functions/CompareRouteOptions';
import { CompareUtilities } from '../functions/CompareUtilities';
import { CompareRouteOptions } from '../functions/CompareRouteOptions';
import { NavigatorError, NavigatorErrorCodes } from '../functions/NavigatorError';
import { NavigationContext } from '../context/NavigationContext';
import { NavigatorUIConstantsContext } from '../context/NavigatorUIConstantsContext';
import { NativeIDKeys } from '../constants/LibraryConstants';
import { LIB_ENV } from '../constants/LibEnv'; //#region - Type Definitions

;
var RouteStatus;

(function (RouteStatus) {
  RouteStatus["INIT"] = "INIT";
  RouteStatus["ROUTE_PUSHING"] = "ROUTE_PUSHING";
  RouteStatus["ROUTE_PUSHED"] = "ROUTE_PUSHED";
  RouteStatus["ROUTE_POPPING"] = "ROUTE_POPPING";
  RouteStatus["ROUTE_POPPED"] = "ROUTE_POPPED";
  RouteStatus["ROUTE_BLURRING"] = "ROUTE_BLURRING";
  RouteStatus["ROUTE_BLURRED"] = "ROUTE_BLURRED";
  RouteStatus["ROUTE_FOCUSING"] = "ROUTE_FOCUSING";
  RouteStatus["ROUTE_FOCUSED"] = "ROUTE_FOCUSED";
  RouteStatus["UNMOUNTED"] = "UNMOUNTED";
})(RouteStatus || (RouteStatus = {}));

;
//#endregion
export class NavigatorRouteView extends React.Component {
  //#region - Property Declarations

  /** is incremented whenever `props.routeProps` changes */
  // references
  //#endregion
  constructor(_props) {
    super(_props);

    _defineProperty(this, "routeStatus", void 0);

    _defineProperty(this, "routePropsUpdateIndex", 0);

    _defineProperty(this, "isPushed", false);

    _defineProperty(this, "_emitter", void 0);

    _defineProperty(this, "_nativeRef", void 0);

    _defineProperty(this, "_navigatorRef", void 0);

    _defineProperty(this, "_routeContentRef", void 0);

    _defineProperty(this, "_routeViewPortalRef", void 0);

    _defineProperty(this, "_routeHeaderWrapperRef", void 0);

    _defineProperty(this, "_routeComponentsWrapperRef", void 0);

    _defineProperty(this, "getRefToNavigator", () => {
      return this.props.getRefToNavigator();
    });

    _defineProperty(this, "getRouteRef", () => {
      return this;
    });

    _defineProperty(this, "getEmitterRef", () => {
      return this._emitter;
    });

    _defineProperty(this, "getPortalRef", () => {
      return this._routeViewPortalRef;
    });

    _defineProperty(this, "getRouteOptions", () => {
      var _this$_routeViewPorta, _ref, _routeOptionsOverride, _ref2, _routeOptionsOverride2, _ref3, _ref4, _routeOptionsOverride3, _ref5, _ref6, _props$transitionConf, _ref7, _ref8, _props$transitionConf2, _ref9, _ref10, _routeOptionsOverride4, _ref11, _routeOptionsOverride5, _ref12, _routeOptionsOverride6, _ref13, _routeOptionsOverride7, _ref14, _routeOptionsOverride8, _ref15, _routeOptionsOverride9, _ref16, _ref17, _routeOptionsOverride10, _ref18, _routeOptionsOverride11, _ref19, _ref20, _routeOptionsOverride12, _ref21, _ref22, _routeOptionsOverride13, _ref23, _routeOptionsOverride14, _ref24, _routeOptionsOverride15, _ref25, _routeOptionsOverride16, _ref26, _routeOptionsOverride17, _ref27, _routeOptionsOverride18;

      const props = this.props;
      const state = this.state;
      const routeOptionsDefault = props.routeOptionsDefault;
      const routeOptionsPortal = state.routeOptionsPortal;
      const routeOptionsOverride = state.routeOptionsOverride;
      const portalProps = (_this$_routeViewPorta = this._routeViewPortalRef) === null || _this$_routeViewPorta === void 0 ? void 0 : _this$_routeViewPorta.props; // check if portal has custom nav bar items

      const hasNavBarLeftItem = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarLeftItem) != null;
      const hasNavBarRightItem = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarRightItem) != null;
      return {
        // ------------------------------------------------------
        // Technically, this whole thing could be done like this:
        // `{...foo, ...bar, ...baz}`
        // but it's less clear/explicit, idk refactor this later.
        // ------------------------------------------------------
        statusBarStyle: (_ref = (_routeOptionsOverride = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.statusBarStyle) !== null && _routeOptionsOverride !== void 0 ? _routeOptionsOverride : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.statusBarStyle) !== null && _ref !== void 0 ? _ref : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.statusBarStyle,
        routeContainerStyle: (_ref2 = (_routeOptionsOverride2 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.routeContainerStyle) !== null && _routeOptionsOverride2 !== void 0 ? _routeOptionsOverride2 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.routeContainerStyle) !== null && _ref2 !== void 0 ? _ref2 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.routeContainerStyle,
        automaticallyAddHorizontalSafeAreaInsets: (_ref3 = (_ref4 = (_routeOptionsOverride3 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.automaticallyAddHorizontalSafeAreaInsets) !== null && _routeOptionsOverride3 !== void 0 ? _routeOptionsOverride3 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.automaticallyAddHorizontalSafeAreaInsets) !== null && _ref4 !== void 0 ? _ref4 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.automaticallyAddHorizontalSafeAreaInsets) !== null && _ref3 !== void 0 ? _ref3 : true,
        // #region - Transition Config |
        // ----------------------------*
        transitionConfigPush: (_ref5 = (_ref6 = (_props$transitionConf = props.transitionConfigPushOverride) !== null && _props$transitionConf !== void 0 ? _props$transitionConf : routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.transitionConfigPush) !== null && _ref6 !== void 0 ? _ref6 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.transitionConfigPush) !== null && _ref5 !== void 0 ? _ref5 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.transitionConfigPush,
        transitionConfigPop: (_ref7 = (_ref8 = (_props$transitionConf2 = props.transitionConfigPopOverride) !== null && _props$transitionConf2 !== void 0 ? _props$transitionConf2 : routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.transitionConfigPop) !== null && _ref8 !== void 0 ? _ref8 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.transitionConfigPop) !== null && _ref7 !== void 0 ? _ref7 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.transitionConfigPop,
        // #endregion -------------*
        // #region - Navbar Config |
        // ------------------------*
        routeTitle: (_ref9 = (_ref10 = (_routeOptionsOverride4 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.routeTitle) !== null && _routeOptionsOverride4 !== void 0 ? _routeOptionsOverride4 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.routeTitle) !== null && _ref10 !== void 0 ? _ref10 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.routeTitle) !== null && _ref9 !== void 0 ? _ref9 : props.routeKey,
        prompt: (_ref11 = (_routeOptionsOverride5 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.prompt) !== null && _routeOptionsOverride5 !== void 0 ? _routeOptionsOverride5 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.prompt) !== null && _ref11 !== void 0 ? _ref11 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.prompt,
        largeTitleDisplayMode: (_ref12 = (_routeOptionsOverride6 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.largeTitleDisplayMode) !== null && _routeOptionsOverride6 !== void 0 ? _routeOptionsOverride6 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.largeTitleDisplayMode) !== null && _ref12 !== void 0 ? _ref12 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.largeTitleDisplayMode,
        searchBarConfig: (_ref13 = (_routeOptionsOverride7 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.searchBarConfig) !== null && _routeOptionsOverride7 !== void 0 ? _routeOptionsOverride7 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.searchBarConfig) !== null && _ref13 !== void 0 ? _ref13 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.searchBarConfig,
        // #endregion ----------------------------------*
        // #region - `NavigationConfigOverride`-related |
        // ---------------------------------------------*
        navBarAppearanceOverride: (_ref14 = (_routeOptionsOverride8 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.navBarAppearanceOverride) !== null && _routeOptionsOverride8 !== void 0 ? _routeOptionsOverride8 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.navBarAppearanceOverride) !== null && _ref14 !== void 0 ? _ref14 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarAppearanceOverride,
        navigationBarVisibility: (_ref15 = (_routeOptionsOverride9 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.navigationBarVisibility) !== null && _routeOptionsOverride9 !== void 0 ? _routeOptionsOverride9 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.navigationBarVisibility) !== null && _ref15 !== void 0 ? _ref15 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navigationBarVisibility,
        allowTouchEventsToPassThroughNavigationBar: (_ref16 = (_ref17 = (_routeOptionsOverride10 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.allowTouchEventsToPassThroughNavigationBar) !== null && _routeOptionsOverride10 !== void 0 ? _routeOptionsOverride10 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.allowTouchEventsToPassThroughNavigationBar) !== null && _ref17 !== void 0 ? _ref17 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.allowTouchEventsToPassThroughNavigationBar) !== null && _ref16 !== void 0 ? _ref16 : false,
        // #endregion ------------------*
        // #region - Navbar Item Config |
        // -----------------------------*
        navBarButtonBackItemConfig: (_ref18 = (_routeOptionsOverride11 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.navBarButtonBackItemConfig) !== null && _routeOptionsOverride11 !== void 0 ? _routeOptionsOverride11 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.navBarButtonBackItemConfig) !== null && _ref18 !== void 0 ? _ref18 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonBackItemConfig,
        navBarButtonLeftItemsConfig: (_ref19 = (_ref20 = (_routeOptionsOverride12 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.navBarButtonLeftItemsConfig) !== null && _routeOptionsOverride12 !== void 0 ? _routeOptionsOverride12 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.navBarButtonLeftItemsConfig) !== null && _ref20 !== void 0 ? _ref20 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonLeftItemsConfig) !== null && _ref19 !== void 0 ? _ref19 : // custom left bar item was set, so we implicitly/automatically
        // create a `type: CUSTOM` nav bar item config...
        hasNavBarLeftItem ? [{
          type: 'CUSTOM'
        }] : undefined,
        navBarButtonRightItemsConfig: (_ref21 = (_ref22 = (_routeOptionsOverride13 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.navBarButtonRightItemsConfig) !== null && _routeOptionsOverride13 !== void 0 ? _routeOptionsOverride13 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.navBarButtonRightItemsConfig) !== null && _ref22 !== void 0 ? _ref22 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonRightItemsConfig) !== null && _ref21 !== void 0 ? _ref21 : // custom right bar item was set, so we implicitly/automatically
        // create a `type: CUSTOM` nav bar item config...
        hasNavBarRightItem ? [{
          type: 'CUSTOM'
        }] : undefined,
        // #endregion ------------------------------*
        // #region - Navbar back button item config |
        // -----------------------------------------*
        leftItemsSupplementBackButton: (_ref23 = (_routeOptionsOverride14 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.leftItemsSupplementBackButton) !== null && _routeOptionsOverride14 !== void 0 ? _routeOptionsOverride14 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.leftItemsSupplementBackButton) !== null && _ref23 !== void 0 ? _ref23 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.leftItemsSupplementBackButton,
        backButtonTitle: (_ref24 = (_routeOptionsOverride15 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.backButtonTitle) !== null && _routeOptionsOverride15 !== void 0 ? _routeOptionsOverride15 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.backButtonTitle) !== null && _ref24 !== void 0 ? _ref24 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.backButtonTitle,
        backButtonDisplayMode: (_ref25 = (_routeOptionsOverride16 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.backButtonDisplayMode) !== null && _routeOptionsOverride16 !== void 0 ? _routeOptionsOverride16 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.backButtonDisplayMode) !== null && _ref25 !== void 0 ? _ref25 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.backButtonDisplayMode,
        hidesBackButton: (_ref26 = (_routeOptionsOverride17 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.hidesBackButton) !== null && _routeOptionsOverride17 !== void 0 ? _routeOptionsOverride17 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.hidesBackButton) !== null && _ref26 !== void 0 ? _ref26 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.hidesBackButton,
        applyBackButtonConfigToCurrentRoute: (_ref27 = (_routeOptionsOverride18 = routeOptionsOverride === null || routeOptionsOverride === void 0 ? void 0 : routeOptionsOverride.applyBackButtonConfigToCurrentRoute) !== null && _routeOptionsOverride18 !== void 0 ? _routeOptionsOverride18 : routeOptionsPortal === null || routeOptionsPortal === void 0 ? void 0 : routeOptionsPortal.applyBackButtonConfigToCurrentRoute) !== null && _ref27 !== void 0 ? _ref27 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.applyBackButtonConfigToCurrentRoute // #endregion

      };
    });

    _defineProperty(this, "getRouteNavigationObject", () => {
      const props = this.props;
      const routeOptions = this.getRouteOptions();
      return {
        routeOptions,
        // pass down route details/data
        routeKey: props.routeKey,
        routeIndex: props.routeIndex,
        routeProps: props.routeProps,
        // pass down navigator commands
        push: this._navigatorRef.push,
        pop: this._navigatorRef.pop,
        popToRoot: this._navigatorRef.popToRoot,
        removeRoute: this._navigatorRef.removeRoute,
        removeRoutes: this._navigatorRef.removeRoutes,
        replaceRoute: this._navigatorRef.replaceRoute,
        insertRoute: this._navigatorRef.insertRoute,
        setRoutes: this._navigatorRef.setRoutes,
        setNavigationBarHidden: this._navigatorRef.setNavigationBarHidden,
        // pass down misc. navigator commands
        sendCustomCommandToNative: this._navigatorRef.sendCustomCommandToNative,
        getNavigatorConstants: this._navigatorRef.getNavigatorConstants,
        getActiveRoutes: this._navigatorRef.getActiveRoutes,
        dismissModal: this._navigatorRef.dismissModal,
        // pass down convenience navigator commands
        replacePreviousRoute: this._navigatorRef.replacePreviousRoute,
        replaceCurrentRoute: this._navigatorRef.replaceCurrentRoute,
        removePreviousRoute: this._navigatorRef.removePreviousRoute,
        removeAllPrevRoutes: this._navigatorRef.removeAllPrevRoutes,
        // navigator route commands
        getRouteOptions: this.getRouteOptions,
        setRouteOptions: this.setRouteOptions,
        setHidesBackButton: this.setHidesBackButton,
        getRouteConstants: this.getRouteConstants,
        // pass down 'get ref' functions
        getRefToRoute: this._handleGetRefToRoute,
        getRefToNavigator: props.getRefToNavigator,
        getRefToNavRouteEmitter: this._handleGetRefToNavRouteEmitter
      };
    });

    _defineProperty(this, "setRouteOptions", async (routeOptionsOverride) => {
      await Helpers.setStateAsync(this, prevState => ({ ...prevState,
        routeOptionsOverride,
        updateIndex: prevState.updateIndex + 1
      }));
    });

    _defineProperty(this, "setPortalRouteOptions", async (routeOptionsPortal) => {
      await Helpers.setStateAsync(this, prevState => ({ ...prevState,
        routeOptionsPortal,
        updateIndex: prevState.updateIndex + 1
      }));
    });

    _defineProperty(this, "setRouteViewPortalRef", ref => {
      var _this$_routeHeaderWra, _this$_routeComponent;

      this._routeViewPortalRef = ref;
      (_this$_routeHeaderWra = this._routeHeaderWrapperRef) === null || _this$_routeHeaderWra === void 0 ? void 0 : _this$_routeHeaderWra.setPortalRef(ref);
      (_this$_routeComponent = this._routeComponentsWrapperRef) === null || _this$_routeComponent === void 0 ? void 0 : _this$_routeComponent.setPortalRef(ref);
      this.setState({
        hasRoutePortal: true
      });
    });

    _defineProperty(this, "setHidesBackButton", async (isHidden, animated) => {
      try {
        if (!RouteViewUtils.isRouteReady(this.routeStatus)) {
          throw new NavigatorError({
            code: NavigatorErrorCodes.libraryError,
            message: "'NavigatorRouteView' is not mounted"
          });
        }

        ;

        if (this._nativeRef == null) {
          throw new NavigatorError({
            code: NavigatorErrorCodes.libraryError,
            message: "'this._nativeRef' is not set"
          });
        }

        ;
        await Helpers.promiseWithTimeout(1000, RNINavigatorRouteViewModule.setHidesBackButton(Helpers.getNativeNodeHandle(this._nativeRef), isHidden, animated));
      } catch (e) {
        throw new NavigatorError(e);
      }

      ;
    });

    _defineProperty(this, "getRouteConstants", async () => {
      try {
        if (!RouteViewUtils.isRouteReady(this.routeStatus)) {
          throw new NavigatorError({
            code: NavigatorErrorCodes.libraryError,
            message: "'NavigatorRouteView' is not mounted"
          });
        }

        ;

        if (this._nativeRef == null) {
          throw new NavigatorError({
            code: NavigatorErrorCodes.libraryError,
            message: "'this._nativeRef' is not set"
          });
        }

        ;
        const result = await Helpers.promiseWithTimeout(1000, RNINavigatorRouteViewModule.getRouteConstants(Helpers.getNativeNodeHandle(this._nativeRef)));
        return result;
      } catch (e) {
        throw new NavigatorError(e);
      }

      ;
    });

    _defineProperty(this, "_handleGetRefToRoute", () => {
      return this;
    });

    _defineProperty(this, "_handleGetRefToNavRouteEmitter", () => {
      return this._emitter;
    });

    _defineProperty(this, "_handleOnRouteWillPop", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteWillPop, event);

      this.routeStatus = RouteStatus.ROUTE_POPPING;
    });

    _defineProperty(this, "_handleOnRouteDidPop", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteDidPop, event);

      this.routeStatus = RouteStatus.ROUTE_POPPED;
    });

    _defineProperty(this, "_handleOnRouteWillPush", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteWillPush, event);

      this.routeStatus = RouteStatus.ROUTE_PUSHING;
    });

    _defineProperty(this, "_handleOnRouteDidPush", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteDidPush, event);

      this.routeStatus = RouteStatus.ROUTE_PUSHED;
    });

    _defineProperty(this, "_handleOnPressNavBarLeftItem", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onPressNavBarLeftItem, event);
    });

    _defineProperty(this, "_handleOnPressNavBarRightItem", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onPressNavBarRightItem, event);
    });

    _defineProperty(this, "_handleOnRouteWillFocus", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteWillFocus, event);

      this.routeStatus = RouteStatus.ROUTE_FOCUSING;
    });

    _defineProperty(this, "_handleOnRouteDidFocus", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteDidFocus, event);

      this.routeStatus = RouteStatus.ROUTE_FOCUSED;
      this.isPushed = true;
    });

    _defineProperty(this, "_handleOnRouteWillBlur", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteWillBlur, event);

      this.routeStatus = RouteStatus.ROUTE_BLURRING;
    });

    _defineProperty(this, "_handleOnRouteDidBlur", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onRouteDidBlur, event);

      this.routeStatus = RouteStatus.ROUTE_BLURRED;
    });

    _defineProperty(this, "_handleOnUpdateSearchResults", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onUpdateSearchResults, event);
    });

    _defineProperty(this, "_handleOnSearchBarCancelButtonClicked", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onSearchBarCancelButtonClicked, event);
    });

    _defineProperty(this, "_handleOnSearchBarSearchButtonClicked", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavigatorRouteViewEvents.onSearchBarSearchButtonClicked, event);
    });

    _defineProperty(this, "_renderRouteContents", navigation => {
      var _this$_routeViewPorta2;

      const props = this.props;
      const routeOptions = navigation.routeOptions;
      const routeContent = props.renderRouteContent();
      const routeContentWithProps = /*#__PURE__*/React.cloneElement(routeContent, {
        navigation,
        // store a ref to this element
        ...(Helpers.isClassComponent(routeContent) && {
          ref: node => {
            this._routeContentRef = node;
          }
        })
      });
      const portalProps = (_this$_routeViewPorta2 = this._routeViewPortalRef) === null || _this$_routeViewPorta2 === void 0 ? void 0 : _this$_routeViewPorta2.props;
      const hasRouteHeader = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderRouteHeader) != null || props.renderRouteHeader != null;
      return /*#__PURE__*/React.createElement(NavigatorUIConstantsContext.Consumer, null, context => /*#__PURE__*/React.createElement(NavigatorRouteContentWrapper, {
        isInFocus: props.isInFocus,
        routePropsUpdateIndex: this.routePropsUpdateIndex,
        routeContainerStyle: routeOptions.routeContainerStyle,
        automaticallyAddHorizontalSafeAreaInsets: routeOptions.automaticallyAddHorizontalSafeAreaInsets,
        safeAreaInsets: context.safeAreaInsets,
        hasRouteHeader: hasRouteHeader
      }, routeContentWithProps, /*#__PURE__*/React.createElement(RouteHeaderWrapper, {
        ref: r => {
          this._routeHeaderWrapperRef = r;
        },
        navigation: navigation,
        getPortalRef: this.getPortalRef
      })));
    });

    this.routeStatus = RouteStatus.INIT;
    this._emitter = new TSEventEmitter();
    this._navigatorRef = _props.getRefToNavigator();
    this.state = {
      updateIndex: 0,
      hasRoutePortal: false,
      routeOptionsOverride: null,
      routeOptionsPortal: null
    };
  }

  shouldComponentUpdate(nextProps, nextState) {
    const prevProps = this.props;
    const prevState = this.state;
    const didRoutePropsChange = !CompareUtilities.unwrapAndShallowCompareObject(prevProps.routeProps, nextProps.routeProps);
    if (didRoutePropsChange) this.routePropsUpdateIndex++; // note: short-circuit eval so:
    // * value types first
    // * complex/nested objects last

    return didRoutePropsChange // props: compare `routeIndex` - routes were re-arranged/sorted
    || prevProps.routeIndex !== nextProps.routeIndex // props: compare focus change
    || prevProps.isInFocus !== nextProps.isInFocus // state: compare `hasRoutePortal` - portal comp. added/removed 
    || prevState.hasRoutePortal !== nextState.hasRoutePortal // state: compare `updateIndex` - force an update
    || prevState.updateIndex !== nextState.updateIndex // props: compare "push transition config"
    || !CompareRouteTransitionPushConfig.unwrapAndCompare(prevProps.transitionConfigPushOverride, nextProps.transitionConfigPushOverride) // props: compare "pop transition config"
    || !CompareRouteTransitionPopConfig.unwrapAndCompare(prevProps.transitionConfigPopOverride, nextProps.transitionConfigPopOverride) // props: compare "default route options"
    || !CompareRouteOptions.unwrapAndCompare(prevProps.routeOptionsDefault, nextProps.routeOptionsDefault);
  }

  componentWillUnmount() {
    //#region - 🐞 DEBUG 🐛
    LIB_ENV.debugLog && console.log("LOG/JS - NavigatorRouteView, componentWillUnmount" + " - routeID: ".concat(this.props.routeID) + " - routeIndex: ".concat(this.props.routeIndex) + " - routeKey: ".concat(this.props.routeKey)); //#endregion

    this.routeStatus = RouteStatus.UNMOUNTED;
  }

  updateRouteComponents() {
    var _this$_routeComponent2, _this$_routeHeaderWra2;

    // don't allow update until it's pushed
    if (!this.isPushed) return;
    (_this$_routeComponent2 = this._routeComponentsWrapperRef) === null || _this$_routeComponent2 === void 0 ? void 0 : _this$_routeComponent2.requestUpdate();
    (_this$_routeHeaderWra2 = this._routeHeaderWrapperRef) === null || _this$_routeHeaderWra2 === void 0 ? void 0 : _this$_routeHeaderWra2.requestUpdate();
  }

  render() {
    const props = this.props;
    const navigation = this.getRouteNavigationObject();
    const routeOptions = navigation.routeOptions; //#region - 🐞 DEBUG 🐛

    LIB_ENV.debugLogRender && console.log("LOG/JS - NavigatorRouteView, render" + " - routeID: ".concat(props.routeID) + " - routeKey: ".concat(props.routeKey) + " - routeKey: ".concat(props.routeIndex)); //#endregion

    return /*#__PURE__*/React.createElement(NavigationContext.Provider, {
      value: {
        navigation,
        navigatorID: props.navigatorID,
        routeID: props.routeID
      }
    }, /*#__PURE__*/React.createElement(RNINavigatorRouteView, _extends({
      style: styles.navigatorRouteView,
      ref: r => {
        this._nativeRef = r;
      },
      routeID: props.routeID,
      nativeID: NativeIDKeys.NavRouteItem,
      routeKey: props.routeKey,
      routeIndex: props.routeIndex // Route Native Events: Push/Pop Events
      ,
      onRouteWillPop: this._handleOnRouteWillPop,
      onRouteDidPop: this._handleOnRouteDidPop,
      onRouteWillPush: this._handleOnRouteWillPush,
      onRouteDidPush: this._handleOnRouteDidPush // Route Native Events: Focus/Blur
      ,
      onRouteWillFocus: this._handleOnRouteWillFocus,
      onRouteDidFocus: this._handleOnRouteDidFocus,
      onRouteWillBlur: this._handleOnRouteWillBlur,
      onRouteDidBlur: this._handleOnRouteDidBlur // Route Native Events: Navbar Item `onPress`
      ,
      onPressNavBarLeftItem: this._handleOnPressNavBarLeftItem,
      onPressNavBarRightItem: this._handleOnPressNavBarRightItem // Route Native Events: Search
      ,
      onUpdateSearchResults: this._handleOnUpdateSearchResults,
      onSearchBarCancelButtonClicked: this._handleOnSearchBarCancelButtonClicked,
      onSearchBarSearchButtonClicked: this._handleOnSearchBarSearchButtonClicked // pass down navbar item config, back button item config, etc.

    }, routeOptions), this._renderRouteContents(navigation), /*#__PURE__*/React.createElement(RouteComponentsWrapper, {
      ref: r => {
        this._routeComponentsWrapperRef = r;
      },
      navigation: navigation,
      getPortalRef: this.getPortalRef // render nav bar items
      ,
      renderNavBarLeftItem: props.renderNavBarLeftItem,
      renderNavBarRightItem: props.renderNavBarRightItem,
      renderNavBarTitleItem: props.renderNavBarTitleItem
    })));
  }

}
;

class RouteViewUtils {
  static isRouteReady(routeStatus) {
    return routeStatus !== RouteStatus.INIT && routeStatus !== RouteStatus.UNMOUNTED;
  }

  static isRouteFocusingOrFocused(routeStatus) {
    return routeStatus === RouteStatus.ROUTE_PUSHING || routeStatus === RouteStatus.ROUTE_PUSHED || routeStatus === RouteStatus.ROUTE_FOCUSING || routeStatus === RouteStatus.ROUTE_FOCUSED;
  }

}

;
const styles = StyleSheet.create({
  navigatorRouteView: { ...StyleSheet.absoluteFillObject,
    // don't show
    opacity: 0
  }
});
//# sourceMappingURL=NavigatorRouteView.js.map