function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { CompareUtilities } from './CompareUtilities';
import { compareColor } from './CompareMisc';

class CompareNavBarItemConfigBase {
  static compare(oldItem, newItem) {
    if (oldItem.type !== newItem.type) return false;

    switch (oldItem.type) {
      case 'SYSTEM_ITEM':
        if (oldItem.systemItem !== newItem.systemItem) return false;
        break;

      case 'TEXT':
        if (oldItem.title !== newItem.title) return false;
        break;

      case 'FIXED_SPACE':
        if (oldItem.width !== newItem.width) return false;
        break;
      // compare `SupportedImageTypes`

      case 'IMAGE_ASSET':
      case 'IMAGE_SYSTEM':
        if (oldItem.imageValue !== newItem.imageValue) return false;
        break;
    }

    ;
    return true;
  }

}

;

class CompareNavBarItemConfigShared {
  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareNavBarItemConfigShared.propertyMap, oldItem, newItem, true);
  }

}

_defineProperty(CompareNavBarItemConfigShared, "propertyMap", {
  // shallow compare
  key: {
    mode: 'shallow'
  },
  width: {
    mode: 'shallow'
  },
  barButtonItemStyle: {
    mode: 'shallow'
  },
  tintColor: {
    mode: 'custom',
    customCompare: compareColor
  },
  possibleTitles: {
    mode: 'shallowArray'
  }
});

;
export class CompareNavBarItemConfig {
  static compare(oldItem, newItem) {
    return CompareNavBarItemConfigBase.compare(oldItem, newItem) && CompareNavBarItemConfigShared.compare(oldItem, newItem) && CompareUtilities.unwrapAndShallowCompareObject(oldItem.backgroundImage, newItem.backgroundImage) && CompareUtilities.unwrapAndShallowCompareObject(oldItem.titlePositionAdjustment, newItem.titlePositionAdjustment);
  }

}
;
export class CompareNavBarBackItemConfig {
  static compare(oldItem, newItem) {
    return CompareNavBarItemConfigBase.compare(oldItem, newItem) && CompareNavBarItemConfigShared.compare(oldItem, newItem);
  }

}
;
export class CompareNavBarItemsConfig {
  static compare(oldItem, newItem) {
    // array size is different, `oldNavBarItem` !== `newBavBarItem`
    if (oldItem.length !== newItem.length) return false;

    for (let i = 0; i < oldItem.length; i++) {
      const oldNavBarItem = oldItem[i];
      const newBavBarItem = newItem[i];

      if (oldNavBarItem == null && newBavBarItem == null) {
        // both items are null/undefined, skip...
        continue;
      } else if (oldNavBarItem == null || newBavBarItem == null) {
        // one of the items are null/undefined, `oldNavBarItem` !== `newBavBarItem`
        return false;
      } else if (oldNavBarItem.type === 'CUSTOM' || newBavBarItem.type === 'CUSTOM') {
        if (oldNavBarItem.type !== newBavBarItem.type) return false;
      } else if (!CompareNavBarItemConfig.compare(oldNavBarItem, newBavBarItem)) {
        return false;
      }

      ;
    }

    ;
    return true;
  }

}
;
//# sourceMappingURL=CompareNavBarItemConfig.js.map