function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

export let NavigatorErrorCodes;

(function (NavigatorErrorCodes) {
  NavigatorErrorCodes["activeRoutesDeSync"] = "activeRoutesDeSync";
  NavigatorErrorCodes["libraryError"] = "libraryError";
  NavigatorErrorCodes["invalidRouteID"] = "invalidRouteID";
  NavigatorErrorCodes["invalidRouteKey"] = "invalidRouteKey";
  NavigatorErrorCodes["invalidRouteIndex"] = "invalidRouteIndex";
  NavigatorErrorCodes["invalidReactTag"] = "invalidReactTag";
  NavigatorErrorCodes["invalidArguments"] = "invalidArguments";
  NavigatorErrorCodes["routeOutOfBounds"] = "routeOutOfBounds";
  NavigatorErrorCodes["invalidProps"] = "invalidProps";
})(NavigatorErrorCodes || (NavigatorErrorCodes = {}));

;
export class NavigatorError extends Error {
  constructor(error) {
    super();

    _defineProperty(this, "name", void 0);

    _defineProperty(this, "message", void 0);

    const errorObj = NavigatorError.parseErrorObject(error);

    if (errorObj != null) {
      // the error is from native
      this.name = errorObj.code;
      this.message = NavigatorError.createErrorString(errorObj);
    } else if (NavigatorError.isErrorObject(error)) {
      // the error is a JS error
      this.name = error.name;
      this.message = error.message;
      this.stack = error.stack;
    } else {
      this.name = 'error';
      this.message = "".concat(error);
    }

    ;
  }

  static isErrorObject(obj) {
    if (typeof obj !== 'object') return false;
    if (obj == null) return false;
    return 'name' in obj || 'message' in obj || 'stack' in obj;
  }

  static isNavigatorErrorObject(obj) {
    return 'code' in obj && 'domain' in obj;
  }

  /// Navigator-related errors thrown from native are JSON strings
  static parseErrorObject(error) {
    const string = error instanceof Error ? error.message : typeof error === 'string' ? error : null;
    if (string == null) return null;
    const indexStart = string.indexOf('{');
    const indexEnd = string.indexOf('}');
    const jsonString = string.substring(indexStart, indexEnd + 1);

    try {
      const obj = JSON.parse(jsonString);
      if (typeof obj !== 'object') return null;
      if (!NavigatorError.isNavigatorErrorObject(obj)) return null;
      return obj;
    } catch {
      return null;
    }

    ;
  }

  static createErrorString(error) {
    const domain = error.domain == null ? 'An error has ocurred' : "".concat(error.domain, " has failed");
    const code = error.domain == null ? '(unknown error code)' : "with error code '".concat(error.code, "'");
    const message = error.message == null ? 'due to an unknown reason' : "because ".concat(error.message);
    const debug = error.debug == null ? '.' : " - Debug: ".concat(error.debug);
    return "".concat(domain, " ").concat(code, " ").concat(message) + debug;
  }

}
;
//# sourceMappingURL=NavigatorError.js.map