import React from 'react';
import { StyleSheet, View } from 'react-native';
import { CompareEdgeInsets } from '../functions/CompareMisc';
import { CompareUtilities } from '../functions/CompareUtilities';
import { NativeIDKeys } from '../constants/LibraryConstants';

/**
 * Wraps the react element returned from `props.renderRouteContent()`
 * render prop in `NavigatorRouteView._renderRouteContents`.
 * * The purpose of this component is to stop unnecessary re-renders from
 *   propagating down to the "route content".
 */
export class NavigatorRouteContentWrapper extends React.Component {
  shouldComponentUpdate(nextProps) {
    const prevProps = this.props; // when `routePropsUpdateIndex` increments, then `routeProps` changed

    const didRoutePropsChange = prevProps.routePropsUpdateIndex !== nextProps.routePropsUpdateIndex; // only update when in focus or the route props changed

    const shouldUpdate = didRoutePropsChange || nextProps.isInFocus;
    return shouldUpdate && ( // compare whether there's a `renderRouteHeader` or not
    prevProps.hasRouteHeader !== nextProps.hasRouteHeader || // compare `automaticallyAddHorizontalSafeAreaInsets`
    prevProps.automaticallyAddHorizontalSafeAreaInsets !== nextProps.automaticallyAddHorizontalSafeAreaInsets // compare `safeAreaInsets`
    || !CompareEdgeInsets.unwrapAndCompare(prevProps.safeAreaInsets, nextProps.safeAreaInsets) || !CompareUtilities.unwrapAndShallowCompareObject(prevProps.routeContainerStyle, nextProps.routeContainerStyle));
  }

  render() {
    var _safeAreaInsets$left, _safeAreaInsets$right;

    const props = this.props;
    const safeAreaInsets = props.safeAreaInsets;
    const routeContainerStyle = { ...(props.automaticallyAddHorizontalSafeAreaInsets && {
        paddingLeft: (_safeAreaInsets$left = safeAreaInsets === null || safeAreaInsets === void 0 ? void 0 : safeAreaInsets.left) !== null && _safeAreaInsets$left !== void 0 ? _safeAreaInsets$left : 0,
        paddingRight: (_safeAreaInsets$right = safeAreaInsets === null || safeAreaInsets === void 0 ? void 0 : safeAreaInsets.right) !== null && _safeAreaInsets$right !== void 0 ? _safeAreaInsets$right : 0
      })
    };
    return /*#__PURE__*/React.createElement(View, {
      style: [styles.routeContentContainer, routeContainerStyle, props.routeContainerStyle],
      nativeID: NativeIDKeys.RouteContent
    }, this.props.children);
  }

}
;
const styles = StyleSheet.create({
  routeContentContainer: { ...StyleSheet.absoluteFillObject,
    overflow: 'hidden'
  }
});
//# sourceMappingURL=NavigatorRouteContentWrapper.js.map