function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { StyleSheet } from 'react-native';
import { NativeIDKeys } from '../constants/LibraryConstants';
import { RNIWrapperView } from '../native_components/RNIWrapperView';

/** 
 * This component is used to hold `NavigatorRouteView`'s navigation bar items, and the
 * other route-related comp. such as the route header.
 * 
 * * This component needs to wrap the route component items so that they can re-render 
 * and update when needed.
 * 
 * * This component receives the route component items from `RouteViewPortal`. Whenever
 * `RouteViewPortal` updates, this comp. will also update, causing the nav. bar items
 * to re-render and update.
 */
export class RouteComponentsWrapper extends React.Component {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "_routeViewPortalRef", void 0);

    _defineProperty(this, "setPortalRef", portalRef => {
      this._routeViewPortalRef = portalRef;
    });

    _defineProperty(this, "requestUpdate", () => {
      var _this$_routeViewPorta;

      const props = this.props;
      const portalProps = (_this$_routeViewPorta = this._routeViewPortalRef) === null || _this$_routeViewPorta === void 0 ? void 0 : _this$_routeViewPorta.props;
      const hasNavBarItem = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarLeftItem) != null || (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarRightItem) != null || (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarTitleItem) != null || props.renderNavBarLeftItem != null || props.renderNavBarRightItem != null || props.renderNavBarTitleItem != null;

      if (hasNavBarItem) {
        this.forceUpdate();
      }

      ;
    });
  }

  componentDidMount() {
    const props = this.props;
    this._routeViewPortalRef = props.getPortalRef();
  }

  render() {
    var _this$_routeViewPorta2, _portalProps$renderNa, _portalProps$renderNa2, _props$renderNavBarLe, _portalProps$renderNa3, _portalProps$renderNa4, _props$renderNavBarRi, _portalProps$renderNa5, _portalProps$renderNa6, _props$renderNavBarTi;

    const props = this.props;
    const portalProps = (_this$_routeViewPorta2 = this._routeViewPortalRef) === null || _this$_routeViewPorta2 === void 0 ? void 0 : _this$_routeViewPorta2.props;
    const navigation = props.navigation;
    const navBarLeftItem = (_portalProps$renderNa = portalProps === null || portalProps === void 0 ? void 0 : (_portalProps$renderNa2 = portalProps.renderNavBarLeftItem) === null || _portalProps$renderNa2 === void 0 ? void 0 : _portalProps$renderNa2.call(portalProps, navigation)) !== null && _portalProps$renderNa !== void 0 ? _portalProps$renderNa : (_props$renderNavBarLe = props.renderNavBarLeftItem) === null || _props$renderNavBarLe === void 0 ? void 0 : _props$renderNavBarLe.call(props, navigation);
    const navBarRightItem = (_portalProps$renderNa3 = portalProps === null || portalProps === void 0 ? void 0 : (_portalProps$renderNa4 = portalProps.renderNavBarRightItem) === null || _portalProps$renderNa4 === void 0 ? void 0 : _portalProps$renderNa4.call(portalProps, navigation)) !== null && _portalProps$renderNa3 !== void 0 ? _portalProps$renderNa3 : (_props$renderNavBarRi = props.renderNavBarRightItem) === null || _props$renderNavBarRi === void 0 ? void 0 : _props$renderNavBarRi.call(props, navigation);
    const navBarTitleItem = (_portalProps$renderNa5 = portalProps === null || portalProps === void 0 ? void 0 : (_portalProps$renderNa6 = portalProps.renderNavBarTitleItem) === null || _portalProps$renderNa6 === void 0 ? void 0 : _portalProps$renderNa6.call(portalProps, navigation)) !== null && _portalProps$renderNa5 !== void 0 ? _portalProps$renderNa5 : (_props$renderNavBarTi = props.renderNavBarTitleItem) === null || _props$renderNavBarTi === void 0 ? void 0 : _props$renderNavBarTi.call(props, navigation);
    return /*#__PURE__*/React.createElement(React.Fragment, null, navBarLeftItem && /*#__PURE__*/React.createElement(RNIWrapperView, {
      style: styles.routeItemContainer,
      nativeID: NativeIDKeys.NavBarLeftItem,
      shouldNotifyComponentWillUnmount: true
    }, navBarLeftItem), navBarRightItem && /*#__PURE__*/React.createElement(RNIWrapperView, {
      style: styles.routeItemContainer,
      nativeID: NativeIDKeys.NavBarRightItem,
      shouldNotifyComponentWillUnmount: true
    }, navBarRightItem), navBarTitleItem && /*#__PURE__*/React.createElement(RNIWrapperView, {
      style: styles.routeItemContainer,
      nativeID: NativeIDKeys.NavBarTitleItem,
      shouldNotifyComponentWillUnmount: true
    }, navBarTitleItem));
  }

}
;
const styles = StyleSheet.create({
  routeItemContainer: {
    position: 'absolute',
    overflow: 'visible'
  }
});
//# sourceMappingURL=RouteComponentsWrapper.js.map