import React from 'react';
import type { NavigationObject } from '../types/NavigationObject';
import type { RouteOptions } from '../types/RouteOptions';
import type { RenderNavItem, RenderRouteContent } from '../types/NavTypes';
import type { RouteTransitionConfig } from '../types/NavigationCommands';
import type { Nullish } from '../types/UtilityTypes';
import { NavigatorRouteViewEventEmitter } from '../types/NavigatorRouteViewEventEmitter';
import type { NavigatorView } from './NavigatorView';
import type { RouteViewPortal } from './RouteViewPortal';
import { RNINavigatorRouteViewProps } from '../native_components/RNINavigatorRouteView';
export interface RouteContentProps<T = object> {
    navigation?: NavigationObject<T>;
}
declare enum RouteStatus {
    INIT = "INIT",
    ROUTE_PUSHING = "ROUTE_PUSHING",
    ROUTE_PUSHED = "ROUTE_PUSHED",
    ROUTE_POPPING = "ROUTE_POPPING",
    ROUTE_POPPED = "ROUTE_POPPED",
    ROUTE_BLURRING = "ROUTE_BLURRING",
    ROUTE_BLURRED = "ROUTE_BLURRED",
    ROUTE_FOCUSING = "ROUTE_FOCUSING",
    ROUTE_FOCUSED = "ROUTE_FOCUSED",
    UNMOUNTED = "UNMOUNTED"
}
export declare type NavigatorRouteViewProps = Pick<RNINavigatorRouteViewProps, 'routeID' | 'routeKey' | 'routeIndex'> & {
    navigatorID: number;
    routeProps: object | null;
    routeOptionsDefault: RouteOptions | null;
    isInFocus: boolean;
    transitionConfigPushOverride: Nullish<RouteTransitionConfig>;
    transitionConfigPopOverride: Nullish<RouteTransitionConfig>;
    getRefToNavigator: () => NavigatorView;
    renderRouteContent: RenderRouteContent;
    renderNavBarLeftItem: RenderNavItem | null;
    renderNavBarRightItem: RenderNavItem | null;
    renderNavBarTitleItem: RenderNavItem | null;
    renderRouteHeader?: RenderNavItem;
};
/** `NavigatorView` comp. state */
declare type NavigatorRouteViewState = {
    updateIndex: number;
    routeOptionsOverride: Nullish<RouteOptions>;
    routeOptionsPortal: Nullish<RouteOptions>;
    hasRoutePortal: boolean;
};
export declare class NavigatorRouteView extends React.Component<NavigatorRouteViewProps, NavigatorRouteViewState> {
    routeStatus: RouteStatus;
    /** is incremented whenever `props.routeProps` changes */
    private routePropsUpdateIndex;
    private isPushed;
    private _emitter;
    private _nativeRef?;
    private _navigatorRef;
    private _routeContentRef?;
    private _routeViewPortalRef?;
    private _routeHeaderWrapperRef?;
    private _routeComponentsWrapperRef?;
    constructor(props: NavigatorRouteViewProps);
    shouldComponentUpdate(nextProps: NavigatorRouteViewProps, nextState: NavigatorRouteViewState): boolean;
    componentWillUnmount(): void;
    updateRouteComponents(): void;
    getRefToNavigator: () => NavigatorView;
    getRouteRef: () => this;
    getEmitterRef: () => NavigatorRouteViewEventEmitter;
    getPortalRef: () => RouteViewPortal | undefined;
    /** Combines all the route configs into one */
    getRouteOptions: () => RouteOptions;
    getRouteNavigationObject: () => NavigationObject;
    setRouteOptions: (routeOptionsOverride: Nullish<Readonly<RouteOptions>>) => Promise<void>;
    setPortalRouteOptions: (routeOptionsPortal: Nullish<Readonly<RouteOptions>>) => Promise<void>;
    setRouteViewPortalRef: (ref: RouteViewPortal) => void;
    setHidesBackButton: (isHidden: boolean, animated: boolean) => Promise<void>;
    getRouteConstants: () => Promise<import("../native_modules/RNINavigatorRouteViewModule").RouteConstantsObject>;
    private _handleGetRefToRoute;
    private _handleGetRefToNavRouteEmitter;
    private _handleOnRouteWillPop;
    private _handleOnRouteDidPop;
    private _handleOnRouteWillPush;
    private _handleOnRouteDidPush;
    private _handleOnPressNavBarLeftItem;
    private _handleOnPressNavBarRightItem;
    private _handleOnRouteWillFocus;
    private _handleOnRouteDidFocus;
    private _handleOnRouteWillBlur;
    private _handleOnRouteDidBlur;
    private _handleOnUpdateSearchResults;
    private _handleOnSearchBarCancelButtonClicked;
    private _handleOnSearchBarSearchButtonClicked;
    private _renderRouteContents;
    render(): JSX.Element;
}
export {};
