import React from 'react';
import type { NavigatorRouteView } from '../components/NavigatorRouteView';
import type { OnRoutePopEvent, OnRoutePushEvent, OnRouteFocusEvent, OnRouteBlurEvent, OnPressNavBarItemEvent, OnUpdateSearchResultsEvent, OnSearchBarCancelButtonClickedEvent, OnSearchBarSearchButtonClickedEvent } from '../types/RNINavigatorRouteViewEvents';
import { NavigatorRouteViewEventEmitter } from '../types/NavigatorRouteViewEventEmitter';
import { NavigationContextProps } from '../context/NavigationContext';
declare type RouteViewEventsProps = {
    onRouteWillPush?: OnRoutePushEvent;
    onRouteDidPush?: OnRoutePushEvent;
    onRouteWillPop?: OnRoutePopEvent;
    onRouteDidPop?: OnRoutePopEvent;
    onRouteWillFocus?: OnRouteFocusEvent;
    onRouteDidFocus?: OnRouteFocusEvent;
    onRouteWillBlur?: OnRouteBlurEvent;
    onRouteDidBlur?: OnRouteBlurEvent;
    onPressNavBarLeftItem?: OnPressNavBarItemEvent;
    onPressNavBarRightItem?: OnPressNavBarItemEvent;
    onUpdateSearchResults?: OnUpdateSearchResultsEvent;
    onSearchBarCancelButtonClicked?: OnSearchBarCancelButtonClickedEvent;
    onSearchBarSearchButtonClicked?: OnSearchBarSearchButtonClickedEvent;
};
/**
 * This component is used to listen to route-related events.
 *
 * This component is used inside a route, and allows you to listen for the events
 * that are related to that route.
 *
 * This component is just a "convenient" wrapper component that subscribes to the
 * parent route component's event emitter. The reference to the event emitter is obtained
 * via context.
 */
export declare class RouteViewEvents extends React.Component<RouteViewEventsProps> {
    static contextType: React.Context<Partial<NavigationContextProps>>;
    routerRef: NavigatorRouteView;
    emitterRef: NavigatorRouteViewEventEmitter;
    constructor(props: RouteViewEventsProps, context: NavigationContextProps);
    componentWillUnmount(): void;
    _handleOnRouteWillPush: OnRoutePushEvent;
    _handleOnRouteDidPush: OnRoutePushEvent;
    _handleOnRouteWillPop: OnRoutePopEvent;
    _handleOnRouteDidPop: OnRoutePopEvent;
    _handleOnRouteWillFocus: OnRouteFocusEvent;
    _handleOnRouteDidFocus: OnRouteFocusEvent;
    _handleOnRouteWillBlur: OnRouteBlurEvent;
    _handleOnRouteDidBlur: OnRouteBlurEvent;
    _handleOnPressNavBarLeftItem: OnPressNavBarItemEvent;
    _handleOnPressNavBarRightItem: OnPressNavBarItemEvent;
    _handleOnUpdateSearchResults: OnUpdateSearchResultsEvent;
    _handleOnSearchBarCancelButtonClicked: OnSearchBarCancelButtonClickedEvent;
    _handleOnSearchBarSearchButtonClicked: OnSearchBarSearchButtonClickedEvent;
    render(): any;
}
export {};
