import type { Point, PointPreset, DynamicColor } from "./MiscTypes";
import type { EnumValueLiteral } from "./UtilityTypes";
/** Object returned by `Image.resolveAssetSource()` */
export declare type ImageResolvedAssetSource = {
    height: number;
    width: number;
    scale: number;
    uri: string;
};
export declare enum ImageTypes {
    IMAGE_ASSET = "IMAGE_ASSET",
    IMAGE_SYSTEM = "IMAGE_SYSTEM",
    IMAGE_REQUIRE = "IMAGE_REQUIRE",
    IMAGE_EMPTY = "IMAGE_EMPTY",
    IMAGE_RECT = "IMAGE_RECT",
    IMAGE_GRADIENT = "IMAGE_GRADIENT"
}
export declare type ImageRectConfig = {
    width: number;
    height: number;
    fillColor: string;
    borderRadius?: number;
};
/** Maps to `UIImage.RenderingMode`*/
export declare type ImageRenderingModes = 'automatic' | 'alwaysOriginal' | 'alwaysTemplate';
/** `UIImage`-related */
export declare type UIImageConfig = {
    tint?: string | DynamicColor;
    renderingMode?: ImageRenderingModes;
};
export declare type ImageSymbolWeight = 'unspecified' | 'ultraLight' | 'thin' | 'light' | 'regular' | 'medium' | 'semibold' | 'bold' | 'heavy' | 'black';
export declare type ImageSymbolScale = 'default' | 'unspecified' | 'small' | 'medium' | 'large';
/** Maps to `UIImage.SymbolConfiguration` */
declare type ImageSystemSymbolConfiguration = {
    pointSize?: number;
    weight?: ImageSymbolWeight;
    scale?: ImageSymbolScale;
} & ({
    /** Requires iOS 15+ */
    hierarchicalColor?: string | DynamicColor;
} | {
    /** Requires iOS 15+ */
    paletteColors?: [string | DynamicColor];
});
export declare type ImageSystemConfig = ImageSystemSymbolConfiguration & {
    systemName: string;
};
export declare type ImageGradientConfig = Partial<Pick<ImageRectConfig, 'width' | 'height' | 'borderRadius'>> & {
    colors: Array<string>;
    locations?: Array<number>;
    startPoint?: Point | PointPreset;
    endPoint?: Point | PointPreset;
    type?: 'axial' | 'conic' | 'radial';
};
export declare type ImageItemConfig = {
    type: EnumValueLiteral<typeof ImageTypes, 'IMAGE_ASSET'>;
    /** The corresponding key of asset item in the asset catalog */
    imageValue: string;
    imageOptions?: UIImageConfig;
} | {
    type: EnumValueLiteral<typeof ImageTypes, 'IMAGE_SYSTEM'>;
    /** The key/name of the SF Symbols system icon */
    imageValue: ImageSystemConfig;
    imageOptions?: UIImageConfig;
} | {
    type: EnumValueLiteral<typeof ImageTypes, 'IMAGE_REQUIRE'>;
    /** Object returned by `Image.resolveAssetSource()` */
    imageValue: ImageResolvedAssetSource;
    imageOptions?: UIImageConfig;
} | {
    type: EnumValueLiteral<typeof ImageTypes, 'IMAGE_EMPTY'>;
} | {
    type: EnumValueLiteral<typeof ImageTypes, 'IMAGE_RECT'>;
    imageValue: ImageRectConfig;
} | {
    type: EnumValueLiteral<typeof ImageTypes, 'IMAGE_GRADIENT'>;
    imageValue: ImageGradientConfig;
};
export declare type ImageItemConfigTypes = ImageItemConfig['type'];
export {};
