/**
 * `RNINavigatorRouteView` Events
 * This file contains all the route-related events and event objects (e.g. push, pop, blur, focus, search, etc).
 */
declare type BaseEventObject = {
    routeID: number;
    routeKey: string;
    routeIndex: number;
};
export declare type OnRoutePushEventObject = {
    nativeEvent: BaseEventObject;
};
export declare type OnRoutePopEventObject = {
    nativeEvent: BaseEventObject & {
        isUserInitiated: boolean;
    };
};
export declare type OnRouteFocusEventObject = {
    nativeEvent: BaseEventObject & {
        isAnimated: boolean;
        isFirstFocus: boolean;
    };
};
export declare type OnRouteBlurEventObject = {
    nativeEvent: BaseEventObject & {
        isAnimated: boolean;
    };
};
export declare type OnPressNavBarItemEventObject = {
    nativeEvent: BaseEventObject & {
        key: string;
        type: string;
        index: string;
    };
};
export declare type OnUpdateSearchResultsEventObject = {
    nativeEvent: BaseEventObject & {
        text?: string;
        isActive: boolean;
    };
};
export declare type OnSearchBarCancelButtonClickedEventObject = {
    nativeEvent: BaseEventObject;
};
export declare type OnSearchBarSearchButtonClickedEventObject = {
    nativeEvent: BaseEventObject & {
        text?: string;
    };
};
export declare type OnRoutePushEvent = (event: Readonly<OnRoutePushEventObject>) => void;
export declare type OnRouteFocusEvent = (event: Readonly<OnRouteFocusEventObject>) => void;
export declare type OnRouteBlurEvent = (event: Readonly<OnRouteBlurEventObject>) => void;
export declare type OnRoutePopEvent = (event: Readonly<OnRoutePopEventObject>) => void;
export declare type OnPressNavBarItemEvent = (event: Readonly<OnPressNavBarItemEventObject>) => void;
export declare type OnUpdateSearchResultsEvent = (event: Readonly<OnUpdateSearchResultsEventObject>) => void;
export declare type OnSearchBarCancelButtonClickedEvent = (event: Readonly<OnSearchBarCancelButtonClickedEventObject>) => void;
export declare type OnSearchBarSearchButtonClickedEvent = (event: Readonly<OnSearchBarSearchButtonClickedEventObject>) => void;
export {};
